from ctypes import c_byte, c_int16, c_long, c_short, c_uint16, c_ushort

BNT_BNCTriggerModes = c_long
NT_BNCModeTrigger = c_long(0x0000)
NT_BNCModeLVOut = c_long(0xFFFF)

BNT_CurrentLimit = c_long
NT_CurrentLimit_100mA = c_long(0x00)
NT_CurrentLimit_250mA = c_long(0x01)
NT_CurrentLimit_500mA = c_long(0x02)

BNT_FeedbackSignalSelection = c_long
NT_FeedbackSignalDC = c_long(0x0000)
NT_FeedbackSignalAC = c_long(0xFFFF)

BNT_OutputLowPassFilter = c_long
NT_OutputFilter_10Hz = c_long(0x0)
NT_OutputFilter_100Hz = c_long(0x1)
NT_OutputFilter_5kHz = c_long(0x2)
NT_OutputFilter_None = c_long(0x3)

ChannelEnableModes = c_int16
ChannelNone = c_int16(0x00)
Channel1Only = c_int16(0x01)
Channel2Only = c_int16(0x02)
Channel3Only = c_int16(0x03)
Channel4Only = c_int16(0x04)
Channels1and2 = c_int16(0x05)
Channels3and4 = c_int16(0x06)

FF_IOModes = c_long
FF_ToggleOnPositiveEdge = c_long(0x01)
FF_SetPositionOnPositiveEdge = c_long(0x02)
FF_OutputHighAtSetPosition = c_long(0x04)
FF_OutputHighWhemMoving = c_long(0x08)

FF_Positions = c_short

FF_SignalModes = c_long
FF_InputButton = c_long(0x01)
FF_InputLogic = c_long(0x02)
FF_InputSwap = c_long(0x04)
FF_OutputLevel = c_long(0x10)
FF_OutputPulse = c_long(0x20)
FF_OutputSwap = c_long(0x40)

HubAnalogueModes = c_short
analogCh1 = c_short()
analogCh2 = c_short()
externaSignalSMA = c_short()

KIM_Channels = c_uint16
Channel1 = c_uint16(1)
Channel2 = c_uint16(2)
Channel3 = c_uint16(2)
Channel4 = c_uint16(4)

KIM_DirectionSense = c_int16
Dir_Disabled = c_int16(0x0)
Dir_Forward = c_int16(0x01)
Dir_Reverse = c_int16(0x02)

KIM_FBSignalMode = c_int16
FB_LimitSwitch = c_int16(0x01)
FB_Encoder = c_int16(0x02)

KIM_JogMode = c_uint16
JogContinuous = c_uint16(0x01)
JogStep = c_uint16(0x02)

KIM_JoysticModes = c_int16
JS_Velocity = c_int16(0x01)
JS_Jog = c_int16(0x02)
JS_GotoPosition = c_int16(0x03)

KIM_JoystickModes = c_short

KIM_LimitSwitchModes = c_int16
Ignore = c_int16(0x01)
SwitchMakes = c_int16(0x02)
SwitchBreaks = c_int16(0x03)
SwitchMakes_HomeOnly = c_int16(0x04)
SwitchBreaks_HomeOnly = c_int16(0x05)

KIM_Stages = c_ushort
Undefined_stage = c_ushort(0)
PIA_stage = c_ushort(1)
PDR_Stage = c_ushort(2)

KIM_TravelDirection = c_byte
Forward = c_byte(0x01)
Reverse = c_byte(0x02)

KIM_TrigModes = c_int16
Trig_Disabled = c_int16(0x00)
Trig_In_GPI = c_int16(0x01)
Trig_InRelativeMove = c_int16(0x02)
Trig_InAbsoluteMove = c_int16(0x03)
Trig_InResetCount = c_int16(0x04)
Trig_Out_GP0 = c_int16(0x0A)
Trig_Out_InMotion = c_int16(0x0B)
Trig_Out_AtMaxVelocity = c_int16(0x0C)
Trig_Out_PosStepFwd = c_int16(0x0D)
Trig_Out_PosStepRev = c_int16(0x0E)
Trig_Out_PosStepBoth = c_int16(0x0F)
Trig_Out_AtFwdLimit = c_int16(0x10)
Trig_Out_AtRevLimit = c_int16(0x11)
Trig_Out_AtEitherLimit = c_int16(0x12)

KIM_TrigPolarities = c_int16
Trig_High = c_int16(0x01)
Trig_Low = c_int16(0x02)

KLDTriggerMode = c_short
disabled = c_short()
input = c_short()
output = c_short()
laserOn = c_short()
interlockEnabled = c_short()
setPointChange = c_short()
highStability = c_short()
lowStability = c_short()

KLD_RAMPUP = c_int16
KLD_RampUpImmediate = c_int16(0)
KLD_RampUpRamped = c_int16(1)

KLD_TrigPolarity = c_ushort
KLD_TrigPol_High = c_ushort(0x01)
KLD_TrigPol_Low = c_ushort(0x02)

KLD_TriggerMode = c_ushort
KLD_Disabled = c_ushort(0)
KLD_Output = c_ushort(0x0a)
KLD_LaserOn = c_ushort(0x0b)
KLD_InterlockEnabled = c_ushort(0x0c)
KLD_SetPointChange = c_ushort(0x0d)
KLD_HighStability = c_ushort(0x0e)
KLD_LowStability = c_ushort(0x0f)
KLD_Input = c_ushort(1)

KLS_OpMode = c_ushort
KLS_ConstantPower = c_ushort(0)
KLS_ConstantCurrent = c_ushort(1)

KLS_Polarity = c_short

KLS_TrigPolarity = c_ushort
KLS_TrigPol_High = c_ushort(0x01)
KLS_TrigPol_Low = c_ushort(0x02)

KLS_TriggerMode = c_short
KLS_Disabled = c_short(0)
KLS_Output = c_short(0x0a)
KLS_LaserOn = c_short(0x0b)
KLS_InterlockEnabled = c_short(0x0c)
KLS_SetPointChange = c_short(0x0d)
KLS_HighStability = c_short(0x0e)
KLS_LowStability = c_short(0x0f)
KLS_Input = c_short(1)

KMOT_TriggerPortMode = c_short
triggerDisabled = c_short()
inputGeneralPurposeLogic = c_short()
inputMoveRelative = c_short()
inputMoveAbsolute = c_short()
inputHomeAction = c_short()
inputStop = c_short()
outputGeneralPurpose = c_short()
ouptputWhileMoving = c_short()
outputAtMaxVelocity = c_short()
outputPredefinedPositionSteps = c_short()
outputTBDMode = c_short()

KMOT_TriggerPortPolarity = c_short
outputHigh = c_short()
ouptutLow = c_short()

KMOT_WheelDirectionSense = c_short

KMOT_WheelMode = c_short
constantVelocity = c_short()
jog = c_short()
moveAbsolute = c_short()

KNA_Channels = c_long
KNA_ChannelUndefined = c_long(0x00)
KNA_Channel1 = c_long(0x01)
KNA_Channel2 = c_long(0x02)

KNA_FeedbackModeTypes = c_short
PZ_ControlModeUndefined = c_short(0)
PZ_OpenLoop = c_short(1)
PZ_CloseLoop = c_short(2)
PZ_OpenLoopSmooth = c_short(3)
PZ_CloseLoopSmooth = c_short(4)

KNA_FeedbackSource = c_short
tiaValue = c_short()
bnc1VRange = c_short()
bnc2VRange = c_short()
bnc5VRange = c_short()
bnc10VRange = c_short()

KNA_HighOutputVoltageRoute = c_short
default = c_short()
extinSMA = c_short()
extoutSMA = c_short()
enableInputboost = c_short()

KNA_HighVoltageRange = c_short
default75V = c_short()
high150V = c_short()

KNA_LowOutputVoltageRoute = c_short
outputIO1connector = c_short()

KNA_LowVoltageRange = c_short
v10V = c_short()

KNA_TIARange = c_short
i5nA = c_short()
i16nA = c_short()
i50nA = c_short()
i166nA = c_short()
i500nA = c_short()
i1p6uA = c_short()
i5uA = c_short()
i16p6uA = c_short()
i50uA = c_short()
i166uA = c_short()
i500uA = c_short()
i1p66mA = c_short()
i5mA = c_short()

KNA_TriggerPolarity = c_short

KNA_TriggerPortMode = c_short
disabled = c_short()
inputLatching = c_short()
inputTracking = c_short()
inputHome = c_short()
output = c_short()
outputTracking = c_short()

KNA_TriggerPortPolarity = c_short
outputHigh = c_short()
OutputLow = c_short()

KNA_WheelAdjustRate = c_short
lowVoltageChangeRate = c_short()
mediumVoltageChangeRate = c_short()
highVoltageChangeRate = c_short()

KPC_HubAnalogueModes = c_short
inputDisabled = c_short()
allHubBays = c_short()
adjacentHubBays = c_short()
fromExternalSMA = c_short()

KPC_IOSettings = c_short

KPC_MonitorOutputMode = c_short

KPC_TriggerPortMode = c_short
disabled = c_short()
inputGeneralPurpose = c_short()
inputRelative = c_short()
inputAbsolute = c_short()
outputGeneralPurpose = c_short()

KPC_TriggerPortPolarity = c_short
highWhenSet = c_short()
lowWhenSet = c_short()

KPZ_TriggerPortMode = c_short
ddisabled = c_short()
inputGeneralPurpose = c_short()
inputRelative = c_short()
inputAbsolute = c_short()
outputGeneralPurpose = c_short()

KPZ_TriggerPortPolarity = c_short
highWhenSet = c_short()
lowWhenSet = c_short()

KPZ_WheelChangeRate = c_int16
KPZ_WM_High = c_int16(0x01)
KPZ_WM_Medium = c_int16(0x02)
KPZ_WM_Low = c_int16(0x03)

KPZ_WheelDirectionSense = c_int16
KPZ_WM_Positive = c_int16(0x01)
KPZ_WM_Negative = c_int16(0x02)

KPZ_WheelMode = c_int16
KPZ_WM_MoveAtVoltage = c_int16(0x01)
KPZ_WM_JogVoltage = c_int16(0x02)
KPZ_WM_SetVoltage = c_int16(0x03)

KSC_TriggerPolarity = c_short

KSC_TriggerPortMode = c_int16
KSC_TrigDisabled = c_int16(0x00)
KSC_TrigIn_GPI = c_int16(0x01)
KSC_TrigOut_GPO = c_int16(0x0A)

KSC_TriggerPortPolarity = c_int16
KSC_TrigPolarityHigh = c_int16(0x01)
KSC_TrigPolarityLow = c_int16(0x02)

KSG_TriggerPortMode = c_int16
KSG_TrigDisabled = c_int16(0x00)
KSG_TrigIn_GPI = c_int16(0x01)
KSG_TrigOut_GPO = c_int16(0x0A)
KSG_TrigOut_LessThanLowerLimit = c_int16(0x0B)
KSG_TrigOut_MoreThanLowerLimit = c_int16(0x0C)
KSG_TrigOut_LessThanUpperLimit = c_int16(0x0D)
KSG_TrigOut_MoreThanUpperLimit = c_int16(0x0E)
KSG_TrigOut_BetweenLimits = c_int16(0x0F)
KSG_TrigOut_OutsideLimits = c_int16(0x10)

KSG_TriggerPortPolarity = c_int16
KSG_TrigPolarityHigh = c_int16(0x01)
KSG_TrigPolarityLow = c_int16(0x02)

KST_Stages = c_short
ZST6 = c_short()
ZST13 = c_short()
ZST25 = c_short()
ZST206 = c_short()
ZST213 = c_short()
ZST225 = c_short()
ZFS206 = c_short()
ZFS213 = c_short()
ZFS225 = c_short()
NR360 = c_short()
PLS_X = c_short()
PLS_XHiRes = c_short()
FW103 = c_short()

LD_DisplayUnits = c_ushort
LD_ILim = c_ushort(0x01)
LD_ILD = c_ushort(0x02)
LD_IPD = c_ushort(0x03)
LD_PLD = c_ushort(0x04)

LD_InputSourceFlags = c_ushort
LD_SoftwareOnly = c_ushort(0)
LD_SoftwareOnly = c_ushort(0x01)
LD_Potentiometer = c_ushort(0x01)
LD_ExternalSignal = c_ushort(0x02)
LD_Potentiometer = c_ushort(0x04)
LD_WheelAndSoftware = c_ushort(0x04)

LD_POLARITY = c_int16
LD_CathodeGrounded = c_int16(1)
LD_AnodeGrounded = c_int16(2)

LD_TIA_RANGES = c_int16
LD_TIA_10uA = c_int16(1)
LD_TIA_1_10uA = c_int16(1)
LD_TIA_100uA = c_int16(2)
LD_TIA_2_100uA = c_int16(2)
LD_TIA_1mA = c_int16(4)
LD_TIA_3_1mA = c_int16(4)
LD_TIA_10mA = c_int16(8)
LD_TIA_4_10mA = c_int16(8)

LS_DisplayUnits = c_ushort
LS_mAmps = c_ushort(0x01)
LS_mWatts = c_ushort(0x02)
LS_mDb = c_ushort(0x03)

LS_InputSourceFlags = c_ushort
LS_SoftwareOnly = c_ushort(0)
LS_ExternalSignal = c_ushort(0x01)
LS_Potentiometer = c_ushort(0x04)
LS_WheelAndSoftware = c_ushort(0x04)

MOD_AuxIOPortMode = c_short
SW = c_short()
ENC = c_short()

MOD_IOPortMode = c_short
digitalInput = c_short()
digitalOutput = c_short()
analogInput = c_short()
analogOutput = c_short()

MOD_IOPortSource = c_short
software = c_short()
motorCh1 = c_short()
motorCh2 = c_short()
motorCh3 = c_short()

MOD_Monitor_Variable = c_short
positionError = c_short()
position = c_short()
motorPhaseACurrent = c_short()
motorPhaseBCurrent = c_short()
motorPhaseCCurrent = c_short()
motorCurrent = c_short()

MOT_ButtonModes = c_short
joggingMode = c_short()
presentMode = c_short()

MOT_CurrentLoopPhases = c_long
phaseA = c_long()
phaseB = c_long()
phaseAandB = c_long()

MOT_DirectionSense = c_short
normal = c_short()
backwards = c_short()

MOT_HomeLimitSwitchDirection = c_short
undefined = c_short()
forward = c_short()
reverse = c_short()

MOT_JogModes = c_short
undefined = c_short()
continuousJog = c_short()
jogOneStep = c_short()

MOT_LimitSwitchModes = c_short
undefined = c_short()
ignore = c_short()
makesOnContact = c_short()
breaksOnContact = c_short()
makesOnContactWhenHoming = c_short()
breaksOnContactWhenHoming = c_short()
reserved = c_short()
ignoreSwapped = c_short()
makesOnContactSwapped = c_short()
breaksOnContactSwapped = c_short()
makesOnContactWhenHomingSwapped = c_short()
breaksOnContactWhenHomingSwapped = c_short()

MOT_LimitSwitchSWModes = c_short
undefined = c_short()
ignore = c_short()
stopImmediately = c_short()
stopProfiled = c_short()
ignoreRotational = c_short()
stopImmediatelyRotational = c_short()
stopProfiledRotational = c_short()

MOT_LimitsSoftwareApproachPolicy = c_short
disallowIllegalMoves = c_short()
allowPartialMoves = c_short()
allowAllMoves = c_short()

MOT_MotorTypes = c_long
notAMotor = c_long()
dcMotor = c_long()
stepperMotor = c_long()
brushlessMotor = c_long()
customMotor = c_long()

MOT_MovementDirections = c_short
quickest = c_short()
forwards = c_short()
reverse = c_short()

MOT_MovementModes = c_short
linearRange = c_short()
rotationalUnlimited = c_short()
rotationalWrapping = c_short()

MOT_PID_LoopMode = c_long
disabled = c_long()
openLoop = c_long()
closedLoop = c_long()

MOT_RasterScanMoveCmd = c_short
start = c_short()
pause = c_short()
stopDisable = c_short()

MOT_RasterScanMovePattern = c_long
flyback = c_long()
fowardReverse = c_long()

MOT_RasterScanMoveTriggerMode = c_long
software = c_long()
xStep = c_long()
yStep = c_long()
xyScan = c_long()
onOff = c_long()

MOT_StopModes = c_short
undefined = c_short()
immediate = c_short()
profiled = c_short()

MOT_TravelDirection = c_short
undefined = c_short()
fowards = c_short()
reverse = c_short()

MOT_TravelModes = c_short
undefined = c_short()
linear = c_short()
rotational = c_short()

MOT_TriggerInputConfigModes = c_short
triggerInDisabled = c_short()
triggerInGeneralPurpose = c_short()
triggerInRelative = c_short()
triggerInAbsolute = c_short()
triggerInHome = c_short()
triggerInStop = c_short()

MOT_TriggerInputSource = c_short
software = c_short()
port1 = c_short()
port2 = c_short()
port3 = c_short()

MOT_TriggerOutputConfigModes = c_short
triggerOutDisabled = c_short()
triggerOutGeneralPurpose = c_short()
triggerOutInMotion = c_short()
triggerOutAtMaxVelocity = c_short()
triggerOutAtPositionStepForward = c_short()
triggerOutAtPositionStepReverse = c_short()
triggerOutAtPositionBoth = c_short()
triggerOutAtForwardLimit = c_short()
triggerOutAtBackwardsLimit = c_short()
triggerOutAtLimit = c_short()

MOT_TriggerPolarity = c_short
high = c_short()
low = c_short()

MOT_TriggerState = c_short
arm = c_short()
cancel = c_short()

MOT_VelocityProfileModes = c_long
trapezoidal = c_long()
sCurve = c_long()

MPC_IOModes = c_long
MPC_ToggleOnPositiveEdge = c_long(0x01)
MPC_SetPositionOnPositiveEdge = c_long(0x02)
MPC_OutputHighAtSetPosition = c_long(0x04)
MPC_OutputHighWhemMoving = c_long(0x08)

MPC_SignalModes = c_long
MPC_InputButton = c_long(0x01)
MPC_InputLogic = c_long(0x02)
MPC_InputSwap = c_long(0x04)
MPC_OutputLevel = c_long(0x10)
MPC_OutputPulse = c_long(0x20)
MPC_OutputSwap = c_long(0x40)

NT_CircleAdjustment = c_long
linear = c_long()
log = c_long()
square = c_long()
cube = c_long()

NT_CircleDiameterMode = c_long
fixed = c_long()
absPower = c_long()
LUT = c_long()

NT_ControlMode = c_long
undefined = c_long()
openLoop = c_long()
closedLoop = c_long()
openLoopSmoothed = c_long()
closedLoopSmoothed = c_long()

NT_FeedbackSource = c_long
undefined = c_long()
tia = c_long()
bnc1V = c_long()
bnc2V = c_long()
bnc5V = c_long()
bnc10V = c_long()

NT_LowPassFrequency = c_long
disabled = c_long()
f1Hz = c_long()
f3Hz = c_long()
f10Hz = c_long()
f30Hz = c_long()
f100Hz = c_long()

NT_Mode = c_long
undefined = c_long()
piezo = c_long()
latched = c_long()
tracking = c_long()
horizontalTracking = c_long()
verticalTracking = c_long()

NT_OddOrEven = c_short
allTIARanges = c_short()
onlyOdd = c_short()
onlyEven = c_short()

NT_OutputVoltageRoute = c_long
smaOnly = c_long()
smaAndHub = c_long()

NT_SignalState = c_short

NT_TIARange = c_long
i5nA = c_long()
i16nA = c_long()
i50nA = c_long()
i166nA = c_long()
i500nA = c_long()
i1p6uA = c_long()
i5uA = c_long()
i16p6uA = c_long()
i50uA = c_long()
i166uA = c_long()
i500uA = c_long()
i1p66mA = c_long()
i5mA = c_long()

NT_TIARangeMode = c_short
automaticCurrent = c_short()
manualCurrent = c_short()
automaticSupplied = c_short()

NT_UnderOrOver = c_long
inRange = c_long()
underRange = c_long()
overRange = c_long()

NT_VoltageRange = c_long
v5V = c_long()
v10V = c_long()

PCC_DerivFilterState = c_long

PCC_DisplayIntensity = c_long

PCC_FeedbackPolarity = c_long

PCC_IOFeedbackSourceDefinition = c_long

PCC_IOOutputBandwidth = c_long

PCC_IOOutputMode = c_long

PCC_NotchFilterChannel = c_long

PCC_NotchFilterState = c_long

PDXC2_TriggerModes = c_uint16
Manual = c_uint16(0x00)
AnalogRising = c_uint16(0x01)
AnalogFalling = c_uint16(0x02)
FixedStepRising = c_uint16(0x03)
FixedStepFalling = c_uint16(0x04)
TwoPositionRising = c_uint16(0x05)
TwoPositionFalling = c_uint16(0x06)

POL_PaddleBits = None

POL_Paddles = c_short

POL_PattleBits = c_short

PPC_DerivFilterState = c_short
DerivFilterOn = c_short(0x01)
DerivFilterOff = c_short(0x02)
DerivFilterOn = c_short(0x1)
DerivFilterOff = c_short(0x2)

PPC_DisplayIntensity = c_short
Bright = c_short(0x01)
Dim = c_short(0x02)
Off = c_short(0x03)

PPC_FeedbackPolarity = c_long
Inverted = c_long(-1)
NonInverted = c_long(0)

PPC_IOControlMode = c_short
SWOnly = c_short(0x00)
ExtBNC = c_short(0x01)
Joystick = c_short(0x02)
JoystickBnc = c_short(0x03)

PPC_IOFeedbackSourceDefinition = c_short
StrainGauge = c_short(0x01)
Capacitive = c_short(0x02)
Optical = c_short(0x03)

PPC_IOOutputBandwidth = c_short
OP_Unfiltered = c_short(0x01)
OP_200Hz = c_short(0x02)

PPC_IOOutputMode = c_short
HV = c_short(0x01)
PosRaw = c_short(0x02)
PosCorrected = c_short(0x03)

PPC_NotchFilterChannel = c_short
NotchFilter1 = c_short(0x01)
NotchFilter2 = c_short(0x02)
NotchFilterBoth = c_short(0x03)

PPC_NotchFilterState = c_short
NotchFilterOn = c_short(0x01)
NotchFilterOff = c_short(0x02)

PZ_AmpOutParameters = c_short

PZ_ControlModeTypes = c_short
undefined = c_short()
openLoop = c_short()
closedLoop = c_short()
openLoopSmoothed = c_short()
closedLoopSmoothed = c_short()

PZ_InputSourceFlags = c_short
softwareOnly = c_short()
externalSignal = c_short()
potentiometer = c_short()
all = c_short()

PZ_JogModes = c_long

PZ_OutputLUTModes = c_short
continuous = c_short()
fixed = c_short()
outputTrigEnabled = c_short()
inputTrigEnabled = c_short()
outpTrigSenseHigh = c_short()
inputTrigSenseHigh = c_short()
outputGated = c_short()
outputTrigRepeated = c_short()

PZ_StageAxisParameters = None

PolarizerParameters = None

QD_FilterEnable = c_long
QD_Undefined = c_long(0)
QD_Enabled = c_long(1)
QD_Disabled = c_long(2)

QD_KPA_TrigModes = c_long
QD_Trig_Disabled = c_long(0x00)
QD_TrigIn_GPI = c_long(0x01)
QD_TrigIn_LoopOpenClose = c_long(0x02)
KD_TrigOut_GPO = c_long(0x0A)
KD_TrigOut_Sum = c_long(0x0B)
KD_TrigOut_Diff = c_long(0x0C)
KD_TrigOut_SumDiff = c_long(0x0D)

QD_KPA_TrigPolarities = c_long
GD_Trig_High = c_long(0x01)
GD_Trig_Low = c_long(0x02)

QD_LowVoltageRoute = c_short
QD_RouteUndefined = c_short(0)
QD_SMAOnly = c_short(1)
QD_HubAndSMA = c_short(2)

QD_OpenLoopHoldValues = c_short
QD_HoldOnZero = c_short(1)
QD_HoldOnLastValue = c_short(2)

QD_OperatingMode = c_short
QD_ModeUndefined = c_short(0)
QD_Monitor = c_short(1)
QD_OpenLoop = c_short(2)
QD_ClosedLoop = c_short(3)
QD_AutoOpenClosedLoop = c_short(4)

SC_OperatingModes = c_byte
SC_Manual = c_byte(0x01)
SC_Single = c_byte(0x02)
SC_Auto = c_byte(0x03)
SC_Triggered = c_byte(0x04)

SC_OperatingStates = c_byte
SC_Active = c_byte(0x01)
SC_Inactive = c_byte(0x02)

SC_SolenoidStates = c_byte
SC_SolenoidOpen = c_byte(0x01)
SC_SolenoidClosed = c_byte(0x02)

TC_DisplayModes = c_ushort
TC_ActualTemperature = c_ushort(0x00)
TC_TargetTemperature = c_ushort(0x01)
TC_TempDifference = c_ushort(0x02)
TC_Current = c_ushort(0x03)

TC_SensorTypes = c_ushort
TC_Transducer = c_ushort(0x00)
TC_TH20kOhm = c_ushort(0x01)
TC_TH200kOhm = c_ushort(0x02)

TIM_ButtonParameters = None

TIM_ButtonsMode = None

TIM_Channels = None

TIM_Direction = None

TIM_DriveOPParameters = None

TIM_JogMode = None

TIM_JogParameters = None

TSG_DisplayModes = c_long

TSG_Display_Modes = c_short
TSG_Undefined = c_short(0)
TSG_Position = c_short(1)
TSG_Voltage = c_short(2)
TSG_Force = c_short(3)

TSG_Hub_Analogue_Modes = c_short
TSG_HubChannel1 = c_short(1)
TSG_HubChannel2 = c_short(2)

TST_Stages = None



from ctypes import (
    POINTER,
    c_bool,
    c_byte,
    c_char,
    c_char_p,
    c_double,
    c_float,
    c_int,
    c_int16,
    c_int32,
    c_int64,
    c_long,
    c_uint,
    c_ulong,
    c_void_p,
    cdll,
    pointer)
from .definitions.safearray import SafeArray
from .definitions.enumerations import (
    KMOT_TriggerPortMode,
    KMOT_TriggerPortPolarity,
    KMOT_WheelDirectionSense,
    KMOT_WheelMode,
    MOT_JogModes,
    MOT_LimitSwitchModes,
    MOT_LimitSwitchSWModes,
    MOT_LimitsSoftwareApproachPolicy,
    MOT_MovementDirections,
    MOT_MovementModes,
    MOT_StopModes,
    MOT_TravelDirection,
    MOT_TravelModes)
from .definitions.structures import (
    KMOT_MMIParams,
    KMOT_TriggerConfig,
    KMOT_TriggerParams,
    MOT_BVC_ScanParams,
    MOT_DC_PIDParameters,
    MOT_HomingParameters,
    MOT_JogParameters,
    MOT_LimitSwitchParameters,
    MOT_VelocityParameters,
    TLI_DeviceInfo,
    TLI_HardwareInformation)
from .definitions.kinesisexception import KinesisException


lib_path = "C:/Program Files/Thorlabs/Kinesis/"
device_manager = cdll.LoadLibrary(
    lib_path + "Thorlabs.MotionControl.DeviceManager.dll")

lib = cdll.LoadLibrary(
    lib_path + "Thorlabs.MotionControl.Benchtop.VoiceCoil.dll")


# Build the DeviceList.
TLI_BuildDeviceList = lib.TLI_BuildDeviceList
TLI_BuildDeviceList.restype = c_short
TLI_BuildDeviceList.argtypes = []


# Initialize a connection to the Simulation Manager, which must already be running.
TLI_InitializeSimulations = lib.TLI_InitializeSimulations
TLI_InitializeSimulations.restype = c_void_p
TLI_InitializeSimulations.argtypes = []


# Determine if the device front panel can be locked.
BVC_CanDeviceLockFrontPanel = lib.BVC_CanDeviceLockFrontPanel
BVC_CanDeviceLockFrontPanel.restype = c_bool
BVC_CanDeviceLockFrontPanel.argtypes = [POINTER(c_char)]


# Can the device perform a Home.
BVC_CanHome = lib.BVC_CanHome
BVC_CanHome.restype = c_bool
BVC_CanHome.argtypes = [POINTER(c_char)]


# Can this device be moved without Homing.
BVC_CanMoveWithoutHomingFirst = lib.BVC_CanMoveWithoutHomingFirst
BVC_CanMoveWithoutHomingFirst.restype = c_bool
BVC_CanMoveWithoutHomingFirst.argtypes = [POINTER(c_char)]


# Check connection.
BVC_CheckConnection = lib.BVC_CheckConnection
BVC_CheckConnection.restype = c_bool
BVC_CheckConnection.argtypes = [POINTER(c_char)]


# Clears the device message queue.
BVC_ClearMessageQueue = lib.BVC_ClearMessageQueue
BVC_ClearMessageQueue.restype = c_void_p
BVC_ClearMessageQueue.argtypes = [POINTER(c_char)]


# Disconnect and close the device.
BVC_Close = lib.BVC_Close
BVC_Close.restype = c_void_p
BVC_Close.argtypes = [POINTER(c_char)]


# Disable the channel so that motor can be moved by hand.
BVC_DisableChannel = lib.BVC_DisableChannel
BVC_DisableChannel.restype = c_short
BVC_DisableChannel.argtypes = [POINTER(c_char)]


# Enable channel for computer control.
BVC_EnableChannel = lib.BVC_EnableChannel
BVC_EnableChannel.restype = c_short
BVC_EnableChannel.argtypes = [POINTER(c_char)]


# Enables the last message monitoring timer.
BVC_EnableLastMsgTimer = lib.BVC_EnableLastMsgTimer
BVC_EnableLastMsgTimer.restype = c_void_p
BVC_EnableLastMsgTimer.argtypes = [POINTER(c_char), c_bool, c_int32]


# Get the backlash distance setting (used to control hysteresis).
BVC_GetBacklash = lib.BVC_GetBacklash
BVC_GetBacklash.restype = c_long
BVC_GetBacklash.argtypes = [POINTER(c_char)]


# Get the DC PID parameters for DC motors used in an algorithm involving calculus.
BVC_GetDCPIDParams = lib.BVC_GetDCPIDParams
BVC_GetDCPIDParams.restype = c_short
BVC_GetDCPIDParams.argtypes = [POINTER(c_char), MOT_DC_PIDParameters]


# Converts a device unit to a real world unit.
BVC_GetDeviceUnitFromRealValue = lib.BVC_GetDeviceUnitFromRealValue
BVC_GetDeviceUnitFromRealValue.restype = c_short
BVC_GetDeviceUnitFromRealValue.argtypes = [POINTER(c_char), c_double, c_int, c_int]


# Gets the digital output bits.
BVC_GetDigitalOutputs = lib.BVC_GetDigitalOutputs
BVC_GetDigitalOutputs.restype = c_byte
BVC_GetDigitalOutputs.argtypes = [POINTER(c_char)]


# Get the Encoder Counter.
BVC_GetEncoderCounter = lib.BVC_GetEncoderCounter
BVC_GetEncoderCounter.restype = c_long
BVC_GetEncoderCounter.argtypes = [POINTER(c_char)]


# Query if the device front panel locked.
BVC_GetFrontPanelLocked = lib.BVC_GetFrontPanelLocked
BVC_GetFrontPanelLocked.restype = c_bool
BVC_GetFrontPanelLocked.argtypes = [POINTER(c_char)]


# Gets the hardware information from the device.
BVC_GetHardwareInfo = lib.BVC_GetHardwareInfo
BVC_GetHardwareInfo.restype = c_short
BVC_GetHardwareInfo.argtypes = [POINTER(c_char)]


# Gets the hardware information in a block.
BVC_GetHardwareInfoBlock = lib.BVC_GetHardwareInfoBlock
BVC_GetHardwareInfoBlock.restype = c_short
BVC_GetHardwareInfoBlock.argtypes = [POINTER(c_char)]


# Get the homing parameters.
BVC_GetHomingParamsBlock = lib.BVC_GetHomingParamsBlock
BVC_GetHomingParamsBlock.restype = c_short
BVC_GetHomingParamsBlock.argtypes = [POINTER(c_char), MOT_HomingParameters]


# Gets the homing velocity.
BVC_GetHomingVelocity = lib.BVC_GetHomingVelocity
BVC_GetHomingVelocity.restype = c_uint
BVC_GetHomingVelocity.argtypes = [POINTER(c_char)]


# Gets the hub bay number this device is fitted to.
BVC_GetHubBay = lib.BVC_GetHubBay
BVC_GetHubBay.restype = POINTER(c_char)
BVC_GetHubBay.argtypes = [POINTER(c_char)]


# Gets the jog mode.
BVC_GetJogMode = lib.BVC_GetJogMode
BVC_GetJogMode.restype = c_short
BVC_GetJogMode.argtypes = [POINTER(c_char), MOT_JogModes, MOT_StopModes]


# Get the jog parameters.
BVC_GetJogParamsBlock = lib.BVC_GetJogParamsBlock
BVC_GetJogParamsBlock.restype = c_short
BVC_GetJogParamsBlock.argtypes = [POINTER(c_char), MOT_JogParameters]


# Gets the distance to move when jogging.
BVC_GetJogStepSize = lib.BVC_GetJogStepSize
BVC_GetJogStepSize.restype = c_uint
BVC_GetJogStepSize.argtypes = [POINTER(c_char)]


# Gets the jog velocity parameters.
BVC_GetJogVelParams = lib.BVC_GetJogVelParams
BVC_GetJogVelParams.restype = c_short
BVC_GetJogVelParams.argtypes = [POINTER(c_char), c_int, c_int]


# Get the LED indicator bits on cube.
BVC_GetLEDswitches = lib.BVC_GetLEDswitches
BVC_GetLEDswitches.restype = c_long
BVC_GetLEDswitches.argtypes = [POINTER(c_char)]


# Gets the limit switch parameters.
BVC_GetLimitSwitchParams = lib.BVC_GetLimitSwitchParams
BVC_GetLimitSwitchParams.restype = c_short
BVC_GetLimitSwitchParams.argtypes = [POINTER(c_char), MOT_LimitSwitchModes, MOT_LimitSwitchModes, c_uint, c_uint, MOT_LimitSwitchSWModes]


# Get the limit switch parameters.
BVC_GetLimitSwitchParamsBlock = lib.BVC_GetLimitSwitchParamsBlock
BVC_GetLimitSwitchParamsBlock.restype = c_short
BVC_GetLimitSwitchParamsBlock.argtypes = [POINTER(c_char), MOT_LimitSwitchParameters]


# Get the MMI Parameters for the Voice Coil Display Interface.
BVC_GetMMIParams = lib.BVC_GetMMIParams
BVC_GetMMIParams.restype = c_short
BVC_GetMMIParams.argtypes = [POINTER(c_char), KMOT_WheelMode, c_int32, c_int32, KMOT_WheelDirectionSense, c_int32, c_int32, c_int16]


# Gets the MMI parameters for the device.
BVC_GetMMIParamsBlock = lib.BVC_GetMMIParamsBlock
BVC_GetMMIParamsBlock.restype = c_short
BVC_GetMMIParamsBlock.argtypes = [POINTER(c_char), KMOT_MMIParams]


# Get the MMI Parameters for the Voice Coil Display Interface.
BVC_GetMMIParamsExt = lib.BVC_GetMMIParamsExt
BVC_GetMMIParamsExt.restype = c_short
BVC_GetMMIParamsExt.argtypes = [POINTER(c_char), KMOT_WheelMode, c_int32, c_int32, KMOT_WheelDirectionSense, c_int32, c_int32, c_int16, c_int16, c_int16]


# Gets the motor stage parameters.
BVC_GetMotorParams = lib.BVC_GetMotorParams
BVC_GetMotorParams.restype = c_short
BVC_GetMotorParams.argtypes = [POINTER(c_char), c_long, c_long, c_float]


# Gets the motor stage parameters.
BVC_GetMotorParamsExt = lib.BVC_GetMotorParamsExt
BVC_GetMotorParamsExt.restype = c_short
BVC_GetMotorParamsExt.argtypes = [POINTER(c_char), c_double, c_double, c_double]


# Gets the absolute minimum and maximum travel range constants for the current stage.
BVC_GetMotorTravelLimits = lib.BVC_GetMotorTravelLimits
BVC_GetMotorTravelLimits.restype = c_short
BVC_GetMotorTravelLimits.argtypes = [POINTER(c_char), c_double, c_double]


# Get the motor travel mode.
BVC_GetMotorTravelMode = lib.BVC_GetMotorTravelMode
BVC_GetMotorTravelMode.restype = MOT_TravelModes
BVC_GetMotorTravelMode.argtypes = [POINTER(c_char)]


# Gets the absolute maximum velocity and acceleration constants for the current stage.
BVC_GetMotorVelocityLimits = lib.BVC_GetMotorVelocityLimits
BVC_GetMotorVelocityLimits.restype = c_short
BVC_GetMotorVelocityLimits.argtypes = [POINTER(c_char), c_double, c_double]


# Gets the move absolute position.
BVC_GetMoveAbsolutePosition = lib.BVC_GetMoveAbsolutePosition
BVC_GetMoveAbsolutePosition.restype = c_int
BVC_GetMoveAbsolutePosition.argtypes = [POINTER(c_char)]


# Gets the move relative distance.
BVC_GetMoveRelativeDistance = lib.BVC_GetMoveRelativeDistance
BVC_GetMoveRelativeDistance.restype = c_int
BVC_GetMoveRelativeDistance.argtypes = [POINTER(c_char)]


# Get the next MessageQueue item.
BVC_GetNextMessage = lib.BVC_GetNextMessage
BVC_GetNextMessage.restype = c_bool
BVC_GetNextMessage.argtypes = [POINTER(c_char), c_long, c_long, c_ulong]


# Get number of positions.
BVC_GetNumberPositions = lib.BVC_GetNumberPositions
BVC_GetNumberPositions.restype = c_int
BVC_GetNumberPositions.argtypes = [POINTER(c_char)]


# Get the current position.
BVC_GetPosition = lib.BVC_GetPosition
BVC_GetPosition.restype = c_int
BVC_GetPosition.argtypes = [POINTER(c_char)]


# Get the Position Counter.
BVC_GetPositionCounter = lib.BVC_GetPositionCounter
BVC_GetPositionCounter.restype = c_long
BVC_GetPositionCounter.argtypes = [POINTER(c_char)]


# Converts a device unit to a real world unit.
BVC_GetRealValueFromDeviceUnit = lib.BVC_GetRealValueFromDeviceUnit
BVC_GetRealValueFromDeviceUnit.restype = c_short
BVC_GetRealValueFromDeviceUnit.argtypes = [POINTER(c_char), c_int, c_double, c_int]


# Gets the BVC Scan parameters.
BVC_GetScanParams = lib.BVC_GetScanParams
BVC_GetScanParams.restype = c_short
BVC_GetScanParams.argtypes = [POINTER(c_char), MOT_BVC_ScanParams]


# Gets the software limits mode.
BVC_GetSoftLimitMode = lib.BVC_GetSoftLimitMode
BVC_GetSoftLimitMode.restype = MOT_LimitsSoftwareApproachPolicy
BVC_GetSoftLimitMode.argtypes = [POINTER(c_char)]


# Gets version number of the device software.
BVC_GetSoftwareVersion = lib.BVC_GetSoftwareVersion
BVC_GetSoftwareVersion.restype = c_ulong
BVC_GetSoftwareVersion.argtypes = [POINTER(c_char)]


# Gets the DC Motor maximum stage position.
BVC_GetStageAxisMaxPos = lib.BVC_GetStageAxisMaxPos
BVC_GetStageAxisMaxPos.restype = c_int
BVC_GetStageAxisMaxPos.argtypes = [POINTER(c_char)]


# Gets the DC Motor minimum stage position.
BVC_GetStageAxisMinPos = lib.BVC_GetStageAxisMinPos
BVC_GetStageAxisMinPos.restype = c_int
BVC_GetStageAxisMinPos.argtypes = [POINTER(c_char)]


# Get the current status bits.
BVC_GetStatusBits = lib.BVC_GetStatusBits
BVC_GetStatusBits.restype = c_ulong
BVC_GetStatusBits.argtypes = [POINTER(c_char)]


# Get the Trigger Configuration Parameters.
BVC_GetTriggerConfigParams = lib.BVC_GetTriggerConfigParams
BVC_GetTriggerConfigParams.restype = c_short
BVC_GetTriggerConfigParams.argtypes = [POINTER(c_char), KMOT_TriggerPortMode, KMOT_TriggerPortPolarity, KMOT_TriggerPortMode, KMOT_TriggerPortPolarity]


# Gets the trigger configuration parameters block.
BVC_GetTriggerConfigParamsBlock = lib.BVC_GetTriggerConfigParamsBlock
BVC_GetTriggerConfigParamsBlock.restype = c_short
BVC_GetTriggerConfigParamsBlock.argtypes = [POINTER(c_char), KMOT_TriggerConfig]


# Get the Trigger Parameters Parameters.
BVC_GetTriggerParamsParams = lib.BVC_GetTriggerParamsParams
BVC_GetTriggerParamsParams.restype = c_short
BVC_GetTriggerParamsParams.argtypes = [POINTER(c_char), c_int32, c_int32, c_int32, c_int32, c_int32, c_int32, c_int32, c_int32]


# Gets the trigger parameters block.
BVC_GetTriggerParamsParamsBlock = lib.BVC_GetTriggerParamsParamsBlock
BVC_GetTriggerParamsParamsBlock.restype = c_short
BVC_GetTriggerParamsParamsBlock.argtypes = [POINTER(c_char), KMOT_TriggerParams]


# Gets the move velocity parameters.
BVC_GetVelParams = lib.BVC_GetVelParams
BVC_GetVelParams.restype = c_short
BVC_GetVelParams.argtypes = [POINTER(c_char), c_int, c_int]


# Get the move velocity parameters.
BVC_GetVelParamsBlock = lib.BVC_GetVelParamsBlock
BVC_GetVelParamsBlock.restype = c_short
BVC_GetVelParamsBlock.argtypes = [POINTER(c_char), MOT_VelocityParameters]


# Queries if the time since the last message has exceeded the lastMsgTimeout set by BVC_EnableLastMsgTimer(char const * serialNo, bool enable, __int32 lastMsgTimeout ).
BVC_HasLastMsgTimerOverrun = lib.BVC_HasLastMsgTimerOverrun
BVC_HasLastMsgTimerOverrun.restype = c_bool
BVC_HasLastMsgTimerOverrun.argtypes = [POINTER(c_char)]


# Home the device.
BVC_Home = lib.BVC_Home
BVC_Home.restype = c_short
BVC_Home.argtypes = [POINTER(c_char)]


# Sends a command to the device to make it identify iteself.
BVC_Identify = lib.BVC_Identify
BVC_Identify.restype = c_void_p
BVC_Identify.argtypes = [POINTER(c_char)]


# Gets the Scanning state.
BVC_IsScanning = lib.BVC_IsScanning
BVC_IsScanning.restype = c_bool
BVC_IsScanning.argtypes = [POINTER(c_char)]


# get whether BVC is scanning.
BVC_IsScanningEnabled = lib.BVC_IsScanningEnabled
BVC_IsScanningEnabled.restype = c_bool
BVC_IsScanningEnabled.argtypes = [POINTER(c_char)]


# Update device with named settings.
BVC_LoadNamedSettings = lib.BVC_LoadNamedSettings
BVC_LoadNamedSettings.restype = c_bool
BVC_LoadNamedSettings.argtypes = [POINTER(c_char), POINTER(c_char)]


# Update device with stored settings.
BVC_LoadSettings = lib.BVC_LoadSettings
BVC_LoadSettings.restype = c_bool
BVC_LoadSettings.argtypes = [POINTER(c_char)]


# Gets the MessageQueue size.
BVC_MessageQueueSize = lib.BVC_MessageQueueSize
BVC_MessageQueueSize.restype = c_int
BVC_MessageQueueSize.argtypes = [POINTER(c_char)]


# Moves the device to the position defined in the SetMoveAbsolute command.
BVC_MoveAbsolute = lib.BVC_MoveAbsolute
BVC_MoveAbsolute.restype = c_short
BVC_MoveAbsolute.argtypes = [POINTER(c_char)]


# Start moving at the current velocity in the specified direction.
BVC_MoveAtVelocity = lib.BVC_MoveAtVelocity
BVC_MoveAtVelocity.restype = c_short
BVC_MoveAtVelocity.argtypes = [POINTER(c_char), MOT_TravelDirection]


# Perform a jog.
BVC_MoveJog = lib.BVC_MoveJog
BVC_MoveJog.restype = c_short
BVC_MoveJog.argtypes = [POINTER(c_char), MOT_TravelDirection]


# Move the motor by a relative amount.
BVC_MoveRelative = lib.BVC_MoveRelative
BVC_MoveRelative.restype = c_short
BVC_MoveRelative.argtypes = [POINTER(c_char), c_int]


# Moves the device by a relative distancce defined by SetMoveRelativeDistance.
BVC_MoveRelativeDistance = lib.BVC_MoveRelativeDistance
BVC_MoveRelativeDistance.restype = c_short
BVC_MoveRelativeDistance.argtypes = [POINTER(c_char)]


# Move the device to the specified position (index).
BVC_MoveToPosition = lib.BVC_MoveToPosition
BVC_MoveToPosition.restype = c_short
BVC_MoveToPosition.argtypes = [POINTER(c_char), c_int]


# Does the device need to be Homed before a move can be performed.
BVC_NeedsHoming = lib.BVC_NeedsHoming
BVC_NeedsHoming.restype = c_bool
BVC_NeedsHoming.argtypes = [POINTER(c_char)]


# Open the device for communications.
BVC_Open = lib.BVC_Open
BVC_Open.restype = c_short
BVC_Open.argtypes = [POINTER(c_char)]


# Update device with stored settings.
BVC_PersistSettings = lib.BVC_PersistSettings
BVC_PersistSettings.restype = c_bool
BVC_PersistSettings.argtypes = [POINTER(c_char)]


# Gets the polling loop duration.
BVC_PollingDuration = lib.BVC_PollingDuration
BVC_PollingDuration.restype = c_long
BVC_PollingDuration.argtypes = [POINTER(c_char)]


# Registers a callback on the message queue.
BVC_RegisterMessageCallback = lib.BVC_RegisterMessageCallback
BVC_RegisterMessageCallback.restype = c_void_p
BVC_RegisterMessageCallback.argtypes = [POINTER(c_char), c_void_p]


# Requests the backlash.
BVC_RequestBacklash = lib.BVC_RequestBacklash
BVC_RequestBacklash.restype = c_short
BVC_RequestBacklash.argtypes = [POINTER(c_char)]


# Request the PID parameters for DC motors used in an algorithm involving calculus.
BVC_RequestDCPIDParams = lib.BVC_RequestDCPIDParams
BVC_RequestDCPIDParams.restype = c_short
BVC_RequestDCPIDParams.argtypes = [POINTER(c_char)]


# Requests the digital output bits.
BVC_RequestDigitalOutputs = lib.BVC_RequestDigitalOutputs
BVC_RequestDigitalOutputs.restype = c_short
BVC_RequestDigitalOutputs.argtypes = [POINTER(c_char)]


# Requests the encoder counter.
BVC_RequestEncoderCounter = lib.BVC_RequestEncoderCounter
BVC_RequestEncoderCounter.restype = c_short
BVC_RequestEncoderCounter.argtypes = [POINTER(c_char)]


# Ask the device if its front panel is locked.
BVC_RequestFrontPanelLocked = lib.BVC_RequestFrontPanelLocked
BVC_RequestFrontPanelLocked.restype = c_short
BVC_RequestFrontPanelLocked.argtypes = [POINTER(c_char)]


# Requests the homing parameters.
BVC_RequestHomingParams = lib.BVC_RequestHomingParams
BVC_RequestHomingParams.restype = c_short
BVC_RequestHomingParams.argtypes = [POINTER(c_char)]


# Requests the jog parameters.
BVC_RequestJogParams = lib.BVC_RequestJogParams
BVC_RequestJogParams.restype = c_short
BVC_RequestJogParams.argtypes = [POINTER(c_char)]


# Request the LED indicator bits on cube.
BVC_RequestLEDswitches = lib.BVC_RequestLEDswitches
BVC_RequestLEDswitches.restype = c_short
BVC_RequestLEDswitches.argtypes = [POINTER(c_char)]


# Requests the limit switch parameters.
BVC_RequestLimitSwitchParams = lib.BVC_RequestLimitSwitchParams
BVC_RequestLimitSwitchParams.restype = c_short
BVC_RequestLimitSwitchParams.argtypes = [POINTER(c_char)]


# Requests the MMI Parameters for the Voice Coil Display Interface.
BVC_RequestMMIparams = lib.BVC_RequestMMIparams
BVC_RequestMMIparams.restype = c_short
BVC_RequestMMIparams.argtypes = [POINTER(c_char)]


# Requests the position of next absolute move.
BVC_RequestMoveAbsolutePosition = lib.BVC_RequestMoveAbsolutePosition
BVC_RequestMoveAbsolutePosition.restype = c_short
BVC_RequestMoveAbsolutePosition.argtypes = [POINTER(c_char)]


# Requests the relative move distance.
BVC_RequestMoveRelativeDistance = lib.BVC_RequestMoveRelativeDistance
BVC_RequestMoveRelativeDistance.restype = c_short
BVC_RequestMoveRelativeDistance.argtypes = [POINTER(c_char)]


# Requests the position trigger parameters.
BVC_RequestPosTriggerParams = lib.BVC_RequestPosTriggerParams
BVC_RequestPosTriggerParams.restype = c_short
BVC_RequestPosTriggerParams.argtypes = [POINTER(c_char)]


# Requests the current position.
BVC_RequestPosition = lib.BVC_RequestPosition
BVC_RequestPosition.restype = c_short
BVC_RequestPosition.argtypes = [POINTER(c_char)]


# Requests the BVC Scan parameters.
BVC_RequestScanParams = lib.BVC_RequestScanParams
BVC_RequestScanParams.restype = c_short
BVC_RequestScanParams.argtypes = [POINTER(c_char)]


# Requests that all settings are download from device.
BVC_RequestSettings = lib.BVC_RequestSettings
BVC_RequestSettings.restype = c_short
BVC_RequestSettings.argtypes = [POINTER(c_char)]


# Request the status bits which identify the current motor state.
BVC_RequestStatusBits = lib.BVC_RequestStatusBits
BVC_RequestStatusBits.restype = c_short
BVC_RequestStatusBits.argtypes = [POINTER(c_char)]


# Requests the Trigger Configuration Parameters.
BVC_RequestTriggerConfigParams = lib.BVC_RequestTriggerConfigParams
BVC_RequestTriggerConfigParams.restype = c_short
BVC_RequestTriggerConfigParams.argtypes = [POINTER(c_char)]


# Requests the velocity parameters.
BVC_RequestVelParams = lib.BVC_RequestVelParams
BVC_RequestVelParams.restype = c_short
BVC_RequestVelParams.argtypes = [POINTER(c_char)]


# Reset the rotation modes for a rotational device.
BVC_ResetRotationModes = lib.BVC_ResetRotationModes
BVC_ResetRotationModes.restype = c_short
BVC_ResetRotationModes.argtypes = [POINTER(c_char)]


# Reset the stage settings to defaults.
BVC_ResetStageToDefaults = lib.BVC_ResetStageToDefaults
BVC_ResetStageToDefaults.restype = c_short
BVC_ResetStageToDefaults.argtypes = [POINTER(c_char)]


# Resume suspended move messages.
BVC_ResumeMoveMessages = lib.BVC_ResumeMoveMessages
BVC_ResumeMoveMessages.restype = c_short
BVC_ResumeMoveMessages.argtypes = [POINTER(c_char)]


# Sets the backlash distance (used to control hysteresis).
BVC_SetBacklash = lib.BVC_SetBacklash
BVC_SetBacklash.restype = c_short
BVC_SetBacklash.argtypes = [POINTER(c_char), c_long]


# Set the PID parameters for DC motors used in an algorithm involving calculus.
BVC_SetDCPIDParams = lib.BVC_SetDCPIDParams
BVC_SetDCPIDParams.restype = c_short
BVC_SetDCPIDParams.argtypes = [POINTER(c_char), MOT_DC_PIDParameters]


# Sets the digital output bits.
BVC_SetDigitalOutputs = lib.BVC_SetDigitalOutputs
BVC_SetDigitalOutputs.restype = c_short
BVC_SetDigitalOutputs.argtypes = [POINTER(c_char), c_byte]


# Sets the motor direction sense.
BVC_SetDirection = lib.BVC_SetDirection
BVC_SetDirection.restype = c_void_p
BVC_SetDirection.argtypes = [POINTER(c_char), c_bool]


# Set the Encoder Counter values.
BVC_SetEncoderCounter = lib.BVC_SetEncoderCounter
BVC_SetEncoderCounter.restype = c_short
BVC_SetEncoderCounter.argtypes = [POINTER(c_char), c_long]


# Sets the device front panel lock state.
BVC_SetFrontPanelLock = lib.BVC_SetFrontPanelLock
BVC_SetFrontPanelLock.restype = c_short
BVC_SetFrontPanelLock.argtypes = [POINTER(c_char), c_bool]


# Set the homing parameters.
BVC_SetHomingParamsBlock = lib.BVC_SetHomingParamsBlock
BVC_SetHomingParamsBlock.restype = c_short
BVC_SetHomingParamsBlock.argtypes = [POINTER(c_char), MOT_HomingParameters]


# Sets the homing velocity.
BVC_SetHomingVelocity = lib.BVC_SetHomingVelocity
BVC_SetHomingVelocity.restype = c_short
BVC_SetHomingVelocity.argtypes = [POINTER(c_char), c_uint]


# Sets the jog mode.
BVC_SetJogMode = lib.BVC_SetJogMode
BVC_SetJogMode.restype = c_short
BVC_SetJogMode.argtypes = [POINTER(c_char), MOT_JogModes, MOT_StopModes]


# Set the jog parameters.
BVC_SetJogParamsBlock = lib.BVC_SetJogParamsBlock
BVC_SetJogParamsBlock.restype = c_short
BVC_SetJogParamsBlock.argtypes = [POINTER(c_char), MOT_JogParameters]


# Sets the distance to move on jogging.
BVC_SetJogStepSize = lib.BVC_SetJogStepSize
BVC_SetJogStepSize.restype = c_short
BVC_SetJogStepSize.argtypes = [POINTER(c_char), c_uint]


# Sets jog velocity parameters.
BVC_SetJogVelParams = lib.BVC_SetJogVelParams
BVC_SetJogVelParams.restype = c_short
BVC_SetJogVelParams.argtypes = [POINTER(c_char), c_int, c_int]


# Set the LED indicator bits on cube.
BVC_SetLEDswitches = lib.BVC_SetLEDswitches
BVC_SetLEDswitches.restype = c_short
BVC_SetLEDswitches.argtypes = [POINTER(c_char), c_long]


# Sets the limit switch parameters.
BVC_SetLimitSwitchParams = lib.BVC_SetLimitSwitchParams
BVC_SetLimitSwitchParams.restype = c_short
BVC_SetLimitSwitchParams.argtypes = [POINTER(c_char), MOT_LimitSwitchModes, MOT_LimitSwitchModes, c_uint, c_uint, MOT_LimitSwitchSWModes]


# Set the limit switch parameters.
BVC_SetLimitSwitchParamsBlock = lib.BVC_SetLimitSwitchParamsBlock
BVC_SetLimitSwitchParamsBlock.restype = c_short
BVC_SetLimitSwitchParamsBlock.argtypes = [POINTER(c_char), MOT_LimitSwitchParameters]


# Sets the software limits mode.
BVC_SetLimitsSoftwareApproachPolicy = lib.BVC_SetLimitsSoftwareApproachPolicy
BVC_SetLimitsSoftwareApproachPolicy.restype = c_void_p
BVC_SetLimitsSoftwareApproachPolicy.argtypes = [POINTER(c_char), MOT_LimitsSoftwareApproachPolicy]


# Set the MMI Parameters for the Voice Coil Display Interface.
BVC_SetMMIParams = lib.BVC_SetMMIParams
BVC_SetMMIParams.restype = c_short
BVC_SetMMIParams.argtypes = [POINTER(c_char), KMOT_WheelMode, c_int32, c_int32, KMOT_WheelDirectionSense, c_int32, c_int32, c_int16]


# Sets the MMI parameters for the device.
BVC_SetMMIParamsBlock = lib.BVC_SetMMIParamsBlock
BVC_SetMMIParamsBlock.restype = c_short
BVC_SetMMIParamsBlock.argtypes = [POINTER(c_char), KMOT_MMIParams]


# Set the MMI Parameters for the Voice Coil Display Interface.
BVC_SetMMIParamsExt = lib.BVC_SetMMIParamsExt
BVC_SetMMIParamsExt.restype = c_short
BVC_SetMMIParamsExt.argtypes = [POINTER(c_char), KMOT_WheelMode, c_int32, c_int32, KMOT_WheelDirectionSense, c_int32, c_int32, c_int16, c_int16, c_int16]


# Sets the motor stage parameters.
BVC_SetMotorParams = lib.BVC_SetMotorParams
BVC_SetMotorParams.restype = c_short
BVC_SetMotorParams.argtypes = [POINTER(c_char), c_long, c_long, c_float]


# Sets the motor stage parameters.
BVC_SetMotorParamsExt = lib.BVC_SetMotorParamsExt
BVC_SetMotorParamsExt.restype = c_short
BVC_SetMotorParamsExt.argtypes = [POINTER(c_char), c_double, c_double, c_double]


# Sets the absolute minimum and maximum travel range constants for the current stage.
BVC_SetMotorTravelLimits = lib.BVC_SetMotorTravelLimits
BVC_SetMotorTravelLimits.restype = c_short
BVC_SetMotorTravelLimits.argtypes = [POINTER(c_char), c_double, c_double]


# Set the motor travel mode.
BVC_SetMotorTravelMode = lib.BVC_SetMotorTravelMode
BVC_SetMotorTravelMode.restype = c_short
BVC_SetMotorTravelMode.argtypes = [POINTER(c_char), MOT_TravelModes]


# Sets the absolute maximum velocity and acceleration constants for the current stage.
BVC_SetMotorVelocityLimits = lib.BVC_SetMotorVelocityLimits
BVC_SetMotorVelocityLimits.restype = c_short
BVC_SetMotorVelocityLimits.argtypes = [POINTER(c_char), c_double, c_double]


# Sets the move absolute position.
BVC_SetMoveAbsolutePosition = lib.BVC_SetMoveAbsolutePosition
BVC_SetMoveAbsolutePosition.restype = c_short
BVC_SetMoveAbsolutePosition.argtypes = [POINTER(c_char), c_int]


# Sets the move relative distance.
BVC_SetMoveRelativeDistance = lib.BVC_SetMoveRelativeDistance
BVC_SetMoveRelativeDistance.restype = c_short
BVC_SetMoveRelativeDistance.argtypes = [POINTER(c_char), c_int]


# Set the Position Counter.
BVC_SetPositionCounter = lib.BVC_SetPositionCounter
BVC_SetPositionCounter.restype = c_short
BVC_SetPositionCounter.argtypes = [POINTER(c_char), c_long]


# Set the rotation modes for a rotational device.
BVC_SetRotationModes = lib.BVC_SetRotationModes
BVC_SetRotationModes.restype = c_short
BVC_SetRotationModes.argtypes = [POINTER(c_char), MOT_MovementModes, MOT_MovementDirections]


# Set the BVC Scan parameters.
BVC_SetScanParams = lib.BVC_SetScanParams
BVC_SetScanParams.restype = c_short
BVC_SetScanParams.argtypes = [POINTER(c_char), MOT_BVC_ScanParams]


# Sets the stage axis position limits.
BVC_SetStageAxisLimits = lib.BVC_SetStageAxisLimits
BVC_SetStageAxisLimits.restype = c_short
BVC_SetStageAxisLimits.argtypes = [POINTER(c_char), c_int, c_int]


# Set the Trigger Configuration Parameters.
BVC_SetTriggerConfigParams = lib.BVC_SetTriggerConfigParams
BVC_SetTriggerConfigParams.restype = c_short
BVC_SetTriggerConfigParams.argtypes = [POINTER(c_char), KMOT_TriggerPortMode, KMOT_TriggerPortPolarity, KMOT_TriggerPortMode, KMOT_TriggerPortPolarity]


# Sets the trigger configuration parameters block.
BVC_SetTriggerConfigParamsBlock = lib.BVC_SetTriggerConfigParamsBlock
BVC_SetTriggerConfigParamsBlock.restype = c_short
BVC_SetTriggerConfigParamsBlock.argtypes = [POINTER(c_char), KMOT_TriggerConfig]


# Set the Trigger Parameters Parameters.
BVC_SetTriggerParamsParams = lib.BVC_SetTriggerParamsParams
BVC_SetTriggerParamsParams.restype = c_short
BVC_SetTriggerParamsParams.argtypes = [POINTER(c_char), c_int32, c_int32, c_int32, c_int32, c_int32, c_int32, c_int32, c_int32]


# Sets the trigger parameters block.
BVC_SetTriggerParamsParamsBlock = lib.BVC_SetTriggerParamsParamsBlock
BVC_SetTriggerParamsParamsBlock.restype = c_short
BVC_SetTriggerParamsParamsBlock.argtypes = [POINTER(c_char), KMOT_TriggerParams]


# Sets the move velocity parameters.
BVC_SetVelParams = lib.BVC_SetVelParams
BVC_SetVelParams.restype = c_short
BVC_SetVelParams.argtypes = [POINTER(c_char), c_int, c_int]


# Set the move velocity parameters.
BVC_SetVelParamsBlock = lib.BVC_SetVelParamsBlock
BVC_SetVelParamsBlock.restype = c_short
BVC_SetVelParamsBlock.argtypes = [POINTER(c_char), MOT_VelocityParameters]


# Starts the internal polling loop which continuously requests position and status.
BVC_StartPolling = lib.BVC_StartPolling
BVC_StartPolling.restype = c_bool
BVC_StartPolling.argtypes = [POINTER(c_char), c_int]


# Starts a scanning.
BVC_StartScanning = lib.BVC_StartScanning
BVC_StartScanning.restype = c_short
BVC_StartScanning.argtypes = [POINTER(c_char)]


# Stop the current move immediately (with risk of losing track of position).
BVC_StopImmediate = lib.BVC_StopImmediate
BVC_StopImmediate.restype = c_short
BVC_StopImmediate.argtypes = [POINTER(c_char)]


# Stops the internal polling loop.
BVC_StopPolling = lib.BVC_StopPolling
BVC_StopPolling.restype = c_void_p
BVC_StopPolling.argtypes = [POINTER(c_char)]


# Stop the current move using the current velocity profile.
BVC_StopProfiled = lib.BVC_StopProfiled
BVC_StopProfiled.restype = c_short
BVC_StopProfiled.argtypes = [POINTER(c_char)]


# Stops a scanning.
BVC_StopScanning = lib.BVC_StopScanning
BVC_StopScanning.restype = c_short
BVC_StopScanning.argtypes = [POINTER(c_char)]


# Suspend automatic messages at ends of moves.
BVC_SuspendMoveMessages = lib.BVC_SuspendMoveMessages
BVC_SuspendMoveMessages.restype = c_short
BVC_SuspendMoveMessages.argtypes = [POINTER(c_char)]


# Gets the time in milliseconds since tha last message was received from the device.
BVC_TimeSinceLastMsgReceived = lib.BVC_TimeSinceLastMsgReceived
BVC_TimeSinceLastMsgReceived.restype = c_bool
BVC_TimeSinceLastMsgReceived.argtypes = [POINTER(c_char), c_int64]


# Wait for next MessageQueue item.
BVC_WaitForMessage = lib.BVC_WaitForMessage
BVC_WaitForMessage.restype = c_bool
BVC_WaitForMessage.argtypes = [POINTER(c_char), c_long, c_long, c_ulong]


# Get the device information from the USB port.
TLI_GetDeviceInfo = lib.TLI_GetDeviceInfo
TLI_GetDeviceInfo.restype = c_short
TLI_GetDeviceInfo.argtypes = [POINTER(c_char), POINTER(c_char), TLI_DeviceInfo]


# Get the entire contents of the device list.
TLI_GetDeviceList = lib.TLI_GetDeviceList
TLI_GetDeviceList.restype = c_short
TLI_GetDeviceList.argtypes = [SafeArray]


# Get the contents of the device list which match the supplied typeID.
TLI_GetDeviceListByType = lib.TLI_GetDeviceListByType
TLI_GetDeviceListByType.restype = c_short
TLI_GetDeviceListByType.argtypes = [SafeArray, c_int]


# Get the contents of the device list which match the supplied typeID.
TLI_GetDeviceListByTypeExt = lib.TLI_GetDeviceListByTypeExt
TLI_GetDeviceListByTypeExt.restype = c_short
TLI_GetDeviceListByTypeExt.argtypes = [POINTER(c_char), c_ulong, c_int]


# Get the contents of the device list which match the supplied typeIDs.
TLI_GetDeviceListByTypes = lib.TLI_GetDeviceListByTypes
TLI_GetDeviceListByTypes.restype = c_short
TLI_GetDeviceListByTypes.argtypes = [SafeArray, c_int, c_int]


# Get the contents of the device list which match the supplied typeIDs.
TLI_GetDeviceListByTypesExt = lib.TLI_GetDeviceListByTypesExt
TLI_GetDeviceListByTypesExt.restype = c_short
TLI_GetDeviceListByTypesExt.argtypes = [POINTER(c_char), c_ulong, c_int, c_int]


# Get the entire contents of the device list.
TLI_GetDeviceListExt = lib.TLI_GetDeviceListExt
TLI_GetDeviceListExt.restype = c_short
TLI_GetDeviceListExt.argtypes = [POINTER(c_char), c_ulong]


# Gets the device list size.
TLI_GetDeviceListSize = lib.TLI_GetDeviceListSize
TLI_GetDeviceListSize.restype = c_short
TLI_GetDeviceListSize.argtypes = []




from ctypes import (
    POINTER,
    c_bool,
    c_char,
    c_char_p,
    c_int,
    c_int32,
    c_int64,
    c_long,
    c_short,
    c_ulong,
    c_void_p,
    cdll,
    pointer)
from .definitions.safearray import SafeArray
from .definitions.enumerations import (
    QD_OperatingMode)
from .definitions.structures import (
    QD_LoopParameters,
    QD_LowPassFilterParameters,
    QD_NotchFilterParameters,
    QD_PIDParameters,
    QD_Position,
    QD_PositionDemandParameters,
    QD_Readings,
    TLI_DeviceInfo,
    TLI_HardwareInformation)
from .definitions.kinesisexception import KinesisException


lib_path = "C:/Program Files/Thorlabs/Kinesis/"
device_manager = cdll.LoadLibrary(
    lib_path + "Thorlabs.MotionControl.DeviceManager.dll")

lib = cdll.LoadLibrary(
    lib_path + "Thorlabs.MotionControl.TCube.Quad.DLL")


# Build the DeviceList.
TLI_BuildDeviceList = lib.TLI_BuildDeviceList
TLI_BuildDeviceList.restype = c_short
TLI_BuildDeviceList.argtypes = []


# Initialize a connection to the Simulation Manager, which must already be running.
TLI_InitializeSimulations = lib.TLI_InitializeSimulations
TLI_InitializeSimulations.restype = c_void_p
TLI_InitializeSimulations.argtypes = []


# Check connection.
QD_CheckConnection = lib.QD_CheckConnection
QD_CheckConnection.restype = c_bool
QD_CheckConnection.argtypes = [POINTER(c_char)]


# Clears the device message queue.
QD_ClearMessageQueue = lib.QD_ClearMessageQueue
QD_ClearMessageQueue.restype = c_void_p
QD_ClearMessageQueue.argtypes = [POINTER(c_char)]


# Disconnect and close the device.
QD_Close = lib.QD_Close
QD_Close.restype = c_void_p
QD_Close.argtypes = [POINTER(c_char)]


# Enables the last message monitoring timer.
QD_EnableLastMsgTimer = lib.QD_EnableLastMsgTimer
QD_EnableLastMsgTimer.restype = c_void_p
QD_EnableLastMsgTimer.argtypes = [POINTER(c_char), c_bool, c_int32]


# Gets position demand output.
QD_GetDemandedPosition = lib.QD_GetDemandedPosition
QD_GetDemandedPosition.restype = c_short
QD_GetDemandedPosition.argtypes = [POINTER(c_char), QD_Position]


# Gets version number of the device firmware.
QD_GetFirmwareVersion = lib.QD_GetFirmwareVersion
QD_GetFirmwareVersion.restype = c_ulong
QD_GetFirmwareVersion.argtypes = [POINTER(c_char)]


# Gets the hardware information from the device.
QD_GetHardwareInfo = lib.QD_GetHardwareInfo
QD_GetHardwareInfo.restype = c_short
QD_GetHardwareInfo.argtypes = [POINTER(c_char)]


# Gets the hardware information in a block.
QD_GetHardwareInfoBlock = lib.QD_GetHardwareInfoBlock
QD_GetHardwareInfoBlock.restype = c_short
QD_GetHardwareInfoBlock.argtypes = [POINTER(c_char)]


# Gets the LED brightness.
QD_GetLEDBrightness = lib.QD_GetLEDBrightness
QD_GetLEDBrightness.restype = c_long
QD_GetLEDBrightness.argtypes = [POINTER(c_char)]


# Gets the feedback loop parameters.
QD_GetLoopPIDparams = lib.QD_GetLoopPIDparams
QD_GetLoopPIDparams.restype = c_short
QD_GetLoopPIDparams.argtypes = [POINTER(c_char), QD_LoopParameters]


# Gets the low pass filter parameters.
QD_GetLowPassFilterparams = lib.QD_GetLowPassFilterparams
QD_GetLowPassFilterparams.restype = c_short
QD_GetLowPassFilterparams.argtypes = [POINTER(c_char), QD_LowPassFilterParameters]


# Get the next MessageQueue item.
QD_GetNextMessage = lib.QD_GetNextMessage
QD_GetNextMessage.restype = c_bool
QD_GetNextMessage.argtypes = [POINTER(c_char), c_long, c_long, c_ulong]


# Gets the notch filter parameters.
QD_GetNotchFilterparams = lib.QD_GetNotchFilterparams
QD_GetNotchFilterparams.restype = c_short
QD_GetNotchFilterparams.argtypes = [POINTER(c_char), QD_NotchFilterParameters]


# Gets the operating mode.
QD_GetOperatingMode = lib.QD_GetOperatingMode
QD_GetOperatingMode.restype = QD_OperatingMode
QD_GetOperatingMode.argtypes = [POINTER(c_char)]


# Gets the feedback loop parameters.
QD_GetPIDparams = lib.QD_GetPIDparams
QD_GetPIDparams.restype = c_short
QD_GetPIDparams.argtypes = [POINTER(c_char), QD_PIDParameters]


# Gets the position demand output parameters.
QD_GetPosDemandParams = lib.QD_GetPosDemandParams
QD_GetPosDemandParams.restype = c_short
QD_GetPosDemandParams.argtypes = [POINTER(c_char), QD_PositionDemandParameters]


# Gets a reading.
QD_GetReading = lib.QD_GetReading
QD_GetReading.restype = c_short
QD_GetReading.argtypes = [POINTER(c_char), QD_Readings]


# Gets version number of the device software.
QD_GetSoftwareVersion = lib.QD_GetSoftwareVersion
QD_GetSoftwareVersion.restype = c_ulong
QD_GetSoftwareVersion.argtypes = [POINTER(c_char)]


# Get the current status bits.
QD_GetStatusBits = lib.QD_GetStatusBits
QD_GetStatusBits.restype = c_ulong
QD_GetStatusBits.argtypes = [POINTER(c_char)]


# Queries if the time since the last message has exceeded the lastMsgTimeout set by QD_EnableLastMsgTimer(char const * serialNo, bool enable, __int32 lastMsgTimeout ).
QD_HasLastMsgTimerOverrun = lib.QD_HasLastMsgTimerOverrun
QD_HasLastMsgTimerOverrun.restype = c_bool
QD_HasLastMsgTimerOverrun.argtypes = [POINTER(c_char)]


# Sends a command to the device to make it identify iteself.
QD_Identify = lib.QD_Identify
QD_Identify.restype = c_void_p
QD_Identify.argtypes = [POINTER(c_char)]


# Update device with named settings.
QD_LoadNamedSettings = lib.QD_LoadNamedSettings
QD_LoadNamedSettings.restype = c_bool
QD_LoadNamedSettings.argtypes = [POINTER(c_char), POINTER(c_char)]


# Update device with stored settings.
QD_LoadSettings = lib.QD_LoadSettings
QD_LoadSettings.restype = c_bool
QD_LoadSettings.argtypes = [POINTER(c_char)]


# Gets the MessageQueue size.
QD_MessageQueueSize = lib.QD_MessageQueueSize
QD_MessageQueueSize.restype = c_int
QD_MessageQueueSize.argtypes = [POINTER(c_char)]


# Open the device for communications.
QD_Open = lib.QD_Open
QD_Open.restype = c_short
QD_Open.argtypes = [POINTER(c_char)]


# persist the devices current settings.
QD_PersistSettings = lib.QD_PersistSettings
QD_PersistSettings.restype = c_bool
QD_PersistSettings.argtypes = [POINTER(c_char)]


# Gets the polling loop duration.
QD_PollingDuration = lib.QD_PollingDuration
QD_PollingDuration.restype = c_long
QD_PollingDuration.argtypes = [POINTER(c_char)]


# Registers a callback on the message queue.
QD_RegisterMessageCallback = lib.QD_RegisterMessageCallback
QD_RegisterMessageCallback.restype = c_void_p
QD_RegisterMessageCallback.argtypes = [POINTER(c_char), c_void_p]


# Requests the LED brightness.
QD_RequestLEDBrightness = lib.QD_RequestLEDBrightness
QD_RequestLEDBrightness.restype = c_short
QD_RequestLEDBrightness.argtypes = [POINTER(c_char)]


# Requests the feedback loop parameters.
QD_RequestLoopPIDparams = lib.QD_RequestLoopPIDparams
QD_RequestLoopPIDparams.restype = c_short
QD_RequestLoopPIDparams.argtypes = [POINTER(c_char)]


# Requests the operating mode.
QD_RequestOperatingMode = lib.QD_RequestOperatingMode
QD_RequestOperatingMode.restype = c_short
QD_RequestOperatingMode.argtypes = [POINTER(c_char)]


# Requests the position demand output parameters.
QD_RequestPosDemandParams = lib.QD_RequestPosDemandParams
QD_RequestPosDemandParams.restype = c_short
QD_RequestPosDemandParams.argtypes = [POINTER(c_char)]


# Request reading.
QD_RequestReading = lib.QD_RequestReading
QD_RequestReading.restype = c_short
QD_RequestReading.argtypes = [POINTER(c_char)]


# Requests that all settings are download from device.
QD_RequestSettings = lib.QD_RequestSettings
QD_RequestSettings.restype = c_short
QD_RequestSettings.argtypes = [POINTER(c_char)]


# Requests the state quantities (actual power, current and status bits).
QD_RequestStatus = lib.QD_RequestStatus
QD_RequestStatus.restype = c_short
QD_RequestStatus.argtypes = [POINTER(c_char)]


# Request the status bits which identify the current device state.
QD_RequestStatusBits = lib.QD_RequestStatusBits
QD_RequestStatusBits.restype = c_short
QD_RequestStatusBits.argtypes = [POINTER(c_char)]


# Sets the LED brightness.
QD_SetLEDBrightness = lib.QD_SetLEDBrightness
QD_SetLEDBrightness.restype = c_short
QD_SetLEDBrightness.argtypes = [POINTER(c_char), c_short]


# Sets the feedback loop parameters.
QD_SetLoopPIDparams = lib.QD_SetLoopPIDparams
QD_SetLoopPIDparams.restype = c_short
QD_SetLoopPIDparams.argtypes = [POINTER(c_char), QD_LoopParameters]


# Sets the low pass filter parameters.
QD_SetLowPassFilterparams = lib.QD_SetLowPassFilterparams
QD_SetLowPassFilterparams.restype = c_short
QD_SetLowPassFilterparams.argtypes = [POINTER(c_char), QD_LowPassFilterParameters]


# Sets the notch filter parameters.
QD_SetNotchFilterparams = lib.QD_SetNotchFilterparams
QD_SetNotchFilterparams.restype = c_short
QD_SetNotchFilterparams.argtypes = [POINTER(c_char), QD_NotchFilterParameters]


# Sets the operating mode.
QD_SetOperatingMode = lib.QD_SetOperatingMode
QD_SetOperatingMode.restype = c_short
QD_SetOperatingMode.argtypes = [POINTER(c_char), QD_OperatingMode, c_bool]


# Sets the feedback loop parameters.
QD_SetPIDparams = lib.QD_SetPIDparams
QD_SetPIDparams.restype = c_short
QD_SetPIDparams.argtypes = [POINTER(c_char), QD_PIDParameters]


# Sets the position demand parameters.
QD_SetPosDemandParams = lib.QD_SetPosDemandParams
QD_SetPosDemandParams.restype = c_short
QD_SetPosDemandParams.argtypes = [POINTER(c_char), QD_PositionDemandParameters]


# Sets position demand output.
QD_SetPosition = lib.QD_SetPosition
QD_SetPosition.restype = c_short
QD_SetPosition.argtypes = [POINTER(c_char), QD_Position]


# Starts the internal polling loop which continuously requests position and status.
QD_StartPolling = lib.QD_StartPolling
QD_StartPolling.restype = c_bool
QD_StartPolling.argtypes = [POINTER(c_char), c_int]


# Stops the internal polling loop.
QD_StopPolling = lib.QD_StopPolling
QD_StopPolling.restype = c_void_p
QD_StopPolling.argtypes = [POINTER(c_char)]


# Gets the time in milliseconds since tha last message was received from the device.
QD_TimeSinceLastMsgReceived = lib.QD_TimeSinceLastMsgReceived
QD_TimeSinceLastMsgReceived.restype = c_bool
QD_TimeSinceLastMsgReceived.argtypes = [POINTER(c_char), c_int64]


# Wait for next MessageQueue item.
QD_WaitForMessage = lib.QD_WaitForMessage
QD_WaitForMessage.restype = c_bool
QD_WaitForMessage.argtypes = [POINTER(c_char), c_long, c_long, c_ulong]


# Get the device information from the USB port.
TLI_GetDeviceInfo = lib.TLI_GetDeviceInfo
TLI_GetDeviceInfo.restype = c_short
TLI_GetDeviceInfo.argtypes = [POINTER(c_char), POINTER(c_char), TLI_DeviceInfo]


# Get the entire contents of the device list.
TLI_GetDeviceList = lib.TLI_GetDeviceList
TLI_GetDeviceList.restype = c_short
TLI_GetDeviceList.argtypes = [SafeArray]


# Get the contents of the device list which match the supplied typeID.
TLI_GetDeviceListByType = lib.TLI_GetDeviceListByType
TLI_GetDeviceListByType.restype = c_short
TLI_GetDeviceListByType.argtypes = [SafeArray, c_int]


# Get the contents of the device list which match the supplied typeID.
TLI_GetDeviceListByTypeExt = lib.TLI_GetDeviceListByTypeExt
TLI_GetDeviceListByTypeExt.restype = c_short
TLI_GetDeviceListByTypeExt.argtypes = [POINTER(c_char), c_ulong, c_int]


# Get the contents of the device list which match the supplied typeIDs.
TLI_GetDeviceListByTypes = lib.TLI_GetDeviceListByTypes
TLI_GetDeviceListByTypes.restype = c_short
TLI_GetDeviceListByTypes.argtypes = [SafeArray, c_int, c_int]


# Get the contents of the device list which match the supplied typeIDs.
TLI_GetDeviceListByTypesExt = lib.TLI_GetDeviceListByTypesExt
TLI_GetDeviceListByTypesExt.restype = c_short
TLI_GetDeviceListByTypesExt.argtypes = [POINTER(c_char), c_ulong, c_int, c_int]


# Get the entire contents of the device list.
TLI_GetDeviceListExt = lib.TLI_GetDeviceListExt
TLI_GetDeviceListExt.restype = c_short
TLI_GetDeviceListExt.argtypes = [POINTER(c_char), c_ulong]


# Gets the device list size.
TLI_GetDeviceListSize = lib.TLI_GetDeviceListSize
TLI_GetDeviceListSize.restype = c_short
TLI_GetDeviceListSize.argtypes = []




from ctypes import (
    POINTER,
    c_bool,
    c_char,
    c_char_p,
    c_int,
    c_int32,
    c_int64,
    c_long,
    c_short,
    c_ulong,
    c_void_p,
    cdll,
    pointer)
from .definitions.safearray import SafeArray
from .definitions.enumerations import (
    TC_DisplayModes,
    TC_SensorTypes)
from .definitions.structures import (
    TC_LoopParameters,
    TLI_DeviceInfo,
    TLI_HardwareInformation)
from .definitions.kinesisexception import KinesisException


lib_path = "C:/Program Files/Thorlabs/Kinesis/"
device_manager = cdll.LoadLibrary(
    lib_path + "Thorlabs.MotionControl.DeviceManager.dll")

lib = cdll.LoadLibrary(
    lib_path + "Thorlabs.MotionControl.TCube.TEC.DLL")


# Build the DeviceList.
TLI_BuildDeviceList = lib.TLI_BuildDeviceList
TLI_BuildDeviceList.restype = c_short
TLI_BuildDeviceList.argtypes = []


# Initialize a connection to the Simulation Manager, which must already be running.
TLI_InitializeSimulations = lib.TLI_InitializeSimulations
TLI_InitializeSimulations.restype = c_void_p
TLI_InitializeSimulations.argtypes = []


# Check connection.
TC_CheckConnection = lib.TC_CheckConnection
TC_CheckConnection.restype = c_bool
TC_CheckConnection.argtypes = [POINTER(c_char)]


# Clears the device message queue.
TC_ClearMessageQueue = lib.TC_ClearMessageQueue
TC_ClearMessageQueue.restype = c_void_p
TC_ClearMessageQueue.argtypes = [POINTER(c_char)]


# Disconnect and close the device.
TC_Close = lib.TC_Close
TC_Close.restype = c_void_p
TC_Close.argtypes = [POINTER(c_char)]


# Disable cube.
TC_Disable = lib.TC_Disable
TC_Disable.restype = c_short
TC_Disable.argtypes = [POINTER(c_char)]


# Tells the device that it is being disconnected.
TC_Disconnect = lib.TC_Disconnect
TC_Disconnect.restype = c_short
TC_Disconnect.argtypes = [POINTER(c_char)]


# Enable cube for computer control.
TC_Enable = lib.TC_Enable
TC_Enable.restype = c_short
TC_Enable.argtypes = [POINTER(c_char)]


# Enables the last message monitoring timer.
TC_EnableLastMsgTimer = lib.TC_EnableLastMsgTimer
TC_EnableLastMsgTimer.restype = c_void_p
TC_EnableLastMsgTimer.argtypes = [POINTER(c_char), c_bool, c_int32]


# Gets the max current limit.
TC_GetCurrentLimit = lib.TC_GetCurrentLimit
TC_GetCurrentLimit.restype = c_long
TC_GetCurrentLimit.argtypes = [POINTER(c_char)]


# Gets current reading.
TC_GetCurrentReading = lib.TC_GetCurrentReading
TC_GetCurrentReading.restype = c_long
TC_GetCurrentReading.argtypes = [POINTER(c_char)]


# Gets version number of the device firmware.
TC_GetFirmwareVersion = lib.TC_GetFirmwareVersion
TC_GetFirmwareVersion.restype = c_ulong
TC_GetFirmwareVersion.argtypes = [POINTER(c_char)]


# Gets the display mode / output mode for the device.
TC_GetHWDisplayMode = lib.TC_GetHWDisplayMode
TC_GetHWDisplayMode.restype = TC_DisplayModes
TC_GetHWDisplayMode.argtypes = [POINTER(c_char)]


# Gets the hardware information from the device.
TC_GetHardwareInfo = lib.TC_GetHardwareInfo
TC_GetHardwareInfo.restype = c_short
TC_GetHardwareInfo.argtypes = [POINTER(c_char)]


# Gets the hardware information in a block.
TC_GetHardwareInfoBlock = lib.TC_GetHardwareInfoBlock
TC_GetHardwareInfoBlock.restype = c_short
TC_GetHardwareInfoBlock.argtypes = [POINTER(c_char)]


# Gets the LED brightness.
TC_GetLEDBrightness = lib.TC_GetLEDBrightness
TC_GetLEDBrightness.restype = c_short
TC_GetLEDBrightness.argtypes = [POINTER(c_char)]


# Get the next MessageQueue item.
TC_GetNextMessage = lib.TC_GetNextMessage
TC_GetNextMessage.restype = c_bool
TC_GetNextMessage.argtypes = [POINTER(c_char), c_long, c_long, c_ulong]


# Gets the sensor type.
TC_GetSensorType = lib.TC_GetSensorType
TC_GetSensorType.restype = TC_SensorTypes
TC_GetSensorType.argtypes = [POINTER(c_char)]


# Gets version number of the device software.
TC_GetSoftwareVersion = lib.TC_GetSoftwareVersion
TC_GetSoftwareVersion.restype = c_ulong
TC_GetSoftwareVersion.argtypes = [POINTER(c_char)]


# Get the current status bits.
TC_GetStatusBits = lib.TC_GetStatusBits
TC_GetStatusBits.restype = c_ulong
TC_GetStatusBits.argtypes = [POINTER(c_char)]


# Gets the temperature loop parameters.
TC_GetTempLoopParams = lib.TC_GetTempLoopParams
TC_GetTempLoopParams.restype = c_short
TC_GetTempLoopParams.argtypes = [POINTER(c_char), TC_LoopParameters]


# Gets temperature reading.
TC_GetTemperatureReading = lib.TC_GetTemperatureReading
TC_GetTemperatureReading.restype = c_short
TC_GetTemperatureReading.argtypes = [POINTER(c_char)]


# Gets the required temperature.
TC_GetTemperatureSet = lib.TC_GetTemperatureSet
TC_GetTemperatureSet.restype = c_short
TC_GetTemperatureSet.argtypes = [POINTER(c_char)]


# Queries if the time since the last message has exceeded the lastMsgTimeout set by TC_EnableLastMsgTimer(char const * serialNo, bool enable, __int32 lastMsgTimeout ).
TC_HasLastMsgTimerOverrun = lib.TC_HasLastMsgTimerOverrun
TC_HasLastMsgTimerOverrun.restype = c_bool
TC_HasLastMsgTimerOverrun.argtypes = [POINTER(c_char)]


# Sends a command to the device to make it identify iteself.
TC_Identify = lib.TC_Identify
TC_Identify.restype = c_void_p
TC_Identify.argtypes = [POINTER(c_char)]


# Update device with named settings.
TC_LoadNamedSettings = lib.TC_LoadNamedSettings
TC_LoadNamedSettings.restype = c_bool
TC_LoadNamedSettings.argtypes = [POINTER(c_char), POINTER(c_char)]


# Update device with stored settings.
TC_LoadSettings = lib.TC_LoadSettings
TC_LoadSettings.restype = c_bool
TC_LoadSettings.argtypes = [POINTER(c_char)]


# Gets the MessageQueue size.
TC_MessageQueueSize = lib.TC_MessageQueueSize
TC_MessageQueueSize.restype = c_int
TC_MessageQueueSize.argtypes = [POINTER(c_char)]


# Open the device for communications.
TC_Open = lib.TC_Open
TC_Open.restype = c_short
TC_Open.argtypes = [POINTER(c_char)]


# persist the devices current settings.
TC_PersistSettings = lib.TC_PersistSettings
TC_PersistSettings.restype = c_bool
TC_PersistSettings.argtypes = [POINTER(c_char)]


# Gets the polling loop duration.
TC_PollingDuration = lib.TC_PollingDuration
TC_PollingDuration.restype = c_long
TC_PollingDuration.argtypes = [POINTER(c_char)]


# Registers a callback on the message queue.
TC_RegisterMessageCallback = lib.TC_RegisterMessageCallback
TC_RegisterMessageCallback.restype = c_void_p
TC_RegisterMessageCallback.argtypes = [POINTER(c_char), c_void_p]


# Requests the device current limit.
TC_RequestCurrentLimit = lib.TC_RequestCurrentLimit
TC_RequestCurrentLimit.restype = c_short
TC_RequestCurrentLimit.argtypes = [POINTER(c_char)]


# Requests the quantity displayed by hardware.
TC_RequestHWDisplayMode = lib.TC_RequestHWDisplayMode
TC_RequestHWDisplayMode.restype = c_short
TC_RequestHWDisplayMode.argtypes = [POINTER(c_char)]


# Requests the LED brightness.
TC_RequestLEDBrightness = lib.TC_RequestLEDBrightness
TC_RequestLEDBrightness.restype = c_short
TC_RequestLEDBrightness.argtypes = [POINTER(c_char)]


# Requests temperature and current readings.
TC_RequestReadings = lib.TC_RequestReadings
TC_RequestReadings.restype = c_short
TC_RequestReadings.argtypes = [POINTER(c_char)]


# Requests the sensor type.
TC_RequestSensorType = lib.TC_RequestSensorType
TC_RequestSensorType.restype = c_short
TC_RequestSensorType.argtypes = [POINTER(c_char)]


# Requests that all settings are download from device.
TC_RequestSettings = lib.TC_RequestSettings
TC_RequestSettings.restype = c_short
TC_RequestSettings.argtypes = [POINTER(c_char)]


# Requests the state quantities (actual temperature, current and status bits).
TC_RequestStatus = lib.TC_RequestStatus
TC_RequestStatus.restype = c_short
TC_RequestStatus.argtypes = [POINTER(c_char)]


# Request the status bits which identify the current device state.
TC_RequestStatusBits = lib.TC_RequestStatusBits
TC_RequestStatusBits.restype = c_short
TC_RequestStatusBits.argtypes = [POINTER(c_char)]


# Requests the temperature loop parameters.
TC_RequestTempLoopParams = lib.TC_RequestTempLoopParams
TC_RequestTempLoopParams.restype = c_short
TC_RequestTempLoopParams.argtypes = [POINTER(c_char)]


# Requests the set temperature.
TC_RequestTemperatureSet = lib.TC_RequestTemperatureSet
TC_RequestTemperatureSet.restype = c_short
TC_RequestTemperatureSet.argtypes = [POINTER(c_char)]


# Reset the device.
TC_Reset = lib.TC_Reset
TC_Reset.restype = c_short
TC_Reset.argtypes = [POINTER(c_char)]


# Sets the max current limit.
TC_SetCurrentLimit = lib.TC_SetCurrentLimit
TC_SetCurrentLimit.restype = c_short
TC_SetCurrentLimit.argtypes = [POINTER(c_char), c_long]


# Sets the display mode / output mode for the device.
TC_SetHWDisplayMode = lib.TC_SetHWDisplayMode
TC_SetHWDisplayMode.restype = c_short
TC_SetHWDisplayMode.argtypes = [POINTER(c_char), TC_DisplayModes]


# Sets the LED brightness.
TC_SetLEDBrightness = lib.TC_SetLEDBrightness
TC_SetLEDBrightness.restype = c_short
TC_SetLEDBrightness.argtypes = [POINTER(c_char), c_short]


# Sets the sensor type.
TC_SetSensorType = lib.TC_SetSensorType
TC_SetSensorType.restype = c_short
TC_SetSensorType.argtypes = [POINTER(c_char), TC_SensorTypes]


# Sets the temperature loop parameters.
TC_SetTempLoopParams = lib.TC_SetTempLoopParams
TC_SetTempLoopParams.restype = c_short
TC_SetTempLoopParams.argtypes = [POINTER(c_char), TC_LoopParameters]


# Sets the required temperature.
TC_SetTemperature = lib.TC_SetTemperature
TC_SetTemperature.restype = c_short
TC_SetTemperature.argtypes = [POINTER(c_char), c_short]


# Starts the internal polling loop which continuously requests position and status.
TC_StartPolling = lib.TC_StartPolling
TC_StartPolling.restype = c_bool
TC_StartPolling.argtypes = [POINTER(c_char), c_int]


# Stops the internal polling loop.
TC_StopPolling = lib.TC_StopPolling
TC_StopPolling.restype = c_void_p
TC_StopPolling.argtypes = [POINTER(c_char)]


# Gets the time in milliseconds since tha last message was received from the device.
TC_TimeSinceLastMsgReceived = lib.TC_TimeSinceLastMsgReceived
TC_TimeSinceLastMsgReceived.restype = c_bool
TC_TimeSinceLastMsgReceived.argtypes = [POINTER(c_char), c_int64]


# Wait for next MessageQueue item.
TC_WaitForMessage = lib.TC_WaitForMessage
TC_WaitForMessage.restype = c_bool
TC_WaitForMessage.argtypes = [POINTER(c_char), c_long, c_long, c_ulong]


# Get the device information from the USB port.
TLI_GetDeviceInfo = lib.TLI_GetDeviceInfo
TLI_GetDeviceInfo.restype = c_short
TLI_GetDeviceInfo.argtypes = [POINTER(c_char), POINTER(c_char), TLI_DeviceInfo]


# Get the entire contents of the device list.
TLI_GetDeviceList = lib.TLI_GetDeviceList
TLI_GetDeviceList.restype = c_short
TLI_GetDeviceList.argtypes = [SafeArray]


# Get the contents of the device list which match the supplied typeID.
TLI_GetDeviceListByType = lib.TLI_GetDeviceListByType
TLI_GetDeviceListByType.restype = c_short
TLI_GetDeviceListByType.argtypes = [SafeArray, c_int]


# Get the contents of the device list which match the supplied typeID.
TLI_GetDeviceListByTypeExt = lib.TLI_GetDeviceListByTypeExt
TLI_GetDeviceListByTypeExt.restype = c_short
TLI_GetDeviceListByTypeExt.argtypes = [POINTER(c_char), c_ulong, c_int]


# Get the contents of the device list which match the supplied typeIDs.
TLI_GetDeviceListByTypes = lib.TLI_GetDeviceListByTypes
TLI_GetDeviceListByTypes.restype = c_short
TLI_GetDeviceListByTypes.argtypes = [SafeArray, c_int, c_int]


# Get the contents of the device list which match the supplied typeIDs.
TLI_GetDeviceListByTypesExt = lib.TLI_GetDeviceListByTypesExt
TLI_GetDeviceListByTypesExt.restype = c_short
TLI_GetDeviceListByTypesExt.argtypes = [POINTER(c_char), c_ulong, c_int, c_int]


# Get the entire contents of the device list.
TLI_GetDeviceListExt = lib.TLI_GetDeviceListExt
TLI_GetDeviceListExt.restype = c_short
TLI_GetDeviceListExt.argtypes = [POINTER(c_char), c_ulong]


# Gets the device list size.
TLI_GetDeviceListSize = lib.TLI_GetDeviceListSize
TLI_GetDeviceListSize.restype = c_short
TLI_GetDeviceListSize.argtypes = []




from ctypes import (
    POINTER,
    c_bool,
    c_char,
    c_char_p,
    c_double,
    c_float,
    c_int,
    c_int32,
    c_int64,
    c_long,
    c_short,
    c_ulong,
    c_void_p,
    cdll,
    pointer)
from .definitions.enumerations import (
    KNA_FeedbackSource,
    KNA_TIARange,
    NT_ControlMode,
    NT_FeedbackSource,
    NT_Mode,
    NT_OddOrEven,
    NT_TIARange,
    NT_TIARangeMode)
from .definitions.structures import (
    KNA_TIARangeParameters,
    KNA_TIAReading,
    NT_CircleDiameterLUT,
    NT_CircleParameters,
    NT_HVComponent,
    NT_LowPassFilterParameters,
    NT_TIARangeParameters,
    NT_TIAReading,
    TLI_HardwareInformation)
from .definitions.kinesisexception import KinesisException


lib_path = "C:/Program Files/Thorlabs/Kinesis/"
device_manager = cdll.LoadLibrary(
    lib_path + "Thorlabs.MotionControl.DeviceManager.dll")

lib = cdll.LoadLibrary(
    lib_path + "Thorlabs.MotionControl.Modular.DLL")


# Enable / Disable the specified channel.
NT_ChannelEnable = lib.NT_ChannelEnable
NT_ChannelEnable.restype = c_short
NT_ChannelEnable.argtypes = [POINTER(c_char), c_long, c_bool]


# Check connection.
NT_CheckConnection = lib.NT_CheckConnection
NT_CheckConnection.restype = c_bool
NT_CheckConnection.argtypes = [POINTER(c_char)]


# clears the message queue.
NT_ClearMessageQueue = lib.NT_ClearMessageQueue
NT_ClearMessageQueue.restype = c_void_p
NT_ClearMessageQueue.argtypes = [POINTER(c_char)]


# Disconnect and close the device.
NT_Close = lib.NT_Close
NT_Close.restype = c_void_p
NT_Close.argtypes = [POINTER(c_char)]


# Tells the device that it is being disconnected.
NT_Disconnect = lib.NT_Disconnect
NT_Disconnect.restype = c_short
NT_Disconnect.argtypes = [POINTER(c_char)]


# Enables the last message monitoring timer.
NT_EnableLastMsgTimer = lib.NT_EnableLastMsgTimer
NT_EnableLastMsgTimer.restype = c_void_p
NT_EnableLastMsgTimer.argtypes = [POINTER(c_char), c_bool, c_int32]


# Gets the scan circle diameter.
NT_GetCircleDiameter = lib.NT_GetCircleDiameter
NT_GetCircleDiameter.restype = c_long
NT_GetCircleDiameter.argtypes = [POINTER(c_char)]


# Gets the scan circle diameter Lookup Table (LUT).
NT_GetCircleDiameterLUT = lib.NT_GetCircleDiameterLUT
NT_GetCircleDiameterLUT.restype = c_short
NT_GetCircleDiameterLUT.argtypes = [POINTER(c_char), NT_CircleDiameterLUT]


# Gets the home position of the scan circle.
NT_GetCircleHomePosition = lib.NT_GetCircleHomePosition
NT_GetCircleHomePosition.restype = c_short
NT_GetCircleHomePosition.argtypes = [POINTER(c_char), NT_HVComponent]


# Gets the scanning circle parameters.
NT_GetCircleParams = lib.NT_GetCircleParams
NT_GetCircleParams.restype = c_short
NT_GetCircleParams.argtypes = [POINTER(c_char), NT_CircleParameters]


# Gets the current scan circle centre position.
NT_GetCirclePosition = lib.NT_GetCirclePosition
NT_GetCirclePosition.restype = c_short
NT_GetCirclePosition.argtypes = [POINTER(c_char), NT_HVComponent]


# Get the NanoTrak control mode.
NT_GetControlMode = lib.NT_GetControlMode
NT_GetControlMode.restype = NT_ControlMode
NT_GetControlMode.argtypes = [POINTER(c_char), c_long]


# Gets the NanoTrak feedback source.
NT_GetFeedbackSource = lib.NT_GetFeedbackSource
NT_GetFeedbackSource.restype = NT_FeedbackSource
NT_GetFeedbackSource.argtypes = [POINTER(c_char)]


# Gets version number of the device firmware.
NT_GetFirmwareVersion = lib.NT_GetFirmwareVersion
NT_GetFirmwareVersion.restype = c_ulong
NT_GetFirmwareVersion.argtypes = [POINTER(c_char)]


# Gets the control loop gain.
NT_GetGain = lib.NT_GetGain
NT_GetGain.restype = c_short
NT_GetGain.argtypes = [POINTER(c_char)]


# Gets the hardware information from the device.
NT_GetHardwareInfo = lib.NT_GetHardwareInfo
NT_GetHardwareInfo.restype = c_short
NT_GetHardwareInfo.argtypes = [POINTER(c_char)]


# Gets the hardware information in a block.
NT_GetHardwareInfoBlock = lib.NT_GetHardwareInfoBlock
NT_GetHardwareInfoBlock.restype = c_short
NT_GetHardwareInfoBlock.argtypes = [POINTER(c_char)]


# Gets the MaxTravel for the Piezos in um.
NT_GetMaxTravel = lib.NT_GetMaxTravel
NT_GetMaxTravel.restype = c_short
NT_GetMaxTravel.argtypes = [POINTER(c_char), c_double, c_double]


# Gets the nanoTrak operating mode.
NT_GetMode = lib.NT_GetMode
NT_GetMode.restype = NT_Mode
NT_GetMode.argtypes = [POINTER(c_char)]


# Gets the NanoTrak channels to (usually) piezos.
NT_GetNTChannels = lib.NT_GetNTChannels
NT_GetNTChannels.restype = c_short
NT_GetNTChannels.argtypes = [POINTER(c_char), c_short, c_short]


# Get the next MessageQueue item.
NT_GetNextMessage = lib.NT_GetNextMessage
NT_GetNextMessage.restype = c_bool
NT_GetNextMessage.argtypes = [POINTER(c_char), c_long, c_long, c_ulong]


# Gets the phase compensation parameters.
NT_GetPhaseCompensationParams = lib.NT_GetPhaseCompensationParams
NT_GetPhaseCompensationParams.restype = c_short
NT_GetPhaseCompensationParams.argtypes = [POINTER(c_char), NT_HVComponent]


# Get the TIA Range Mode and OddEven mode.
NT_GetRangeMode = lib.NT_GetRangeMode
NT_GetRangeMode.restype = c_short
NT_GetRangeMode.argtypes = [POINTER(c_char), NT_TIARangeMode, NT_OddOrEven]


# Gets a reading.
NT_GetReading = lib.NT_GetReading
NT_GetReading.restype = c_short
NT_GetReading.argtypes = [POINTER(c_char), NT_TIAReading, KNA_TIAReading]


# Gets the NanoTrak signal state.
NT_GetSignalState = lib.NT_GetSignalState
NT_GetSignalState.restype = NT_SignalState
NT_GetSignalState.argtypes = [POINTER(c_char)]


# Gets version number of the device software.
NT_GetSoftwareVersion = lib.NT_GetSoftwareVersion
NT_GetSoftwareVersion.restype = c_ulong
NT_GetSoftwareVersion.argtypes = [POINTER(c_char)]


# Get the current status bits.
NT_GetStatusBits = lib.NT_GetStatusBits
NT_GetStatusBits.restype = c_ulong
NT_GetStatusBits.argtypes = [POINTER(c_char)]


# Gets the TIA long pass filter parameters.
NT_GetTIALPFilterParams = lib.NT_GetTIALPFilterParams
NT_GetTIALPFilterParams.restype = c_short
NT_GetTIALPFilterParams.argtypes = [POINTER(c_char), NT_LowPassFilterParameters]


# Gets the TIA range.
NT_GetTIARange = lib.NT_GetTIARange
NT_GetTIARange.restype = NT_TIARange
NT_GetTIARange.argtypes = [POINTER(c_char)]


# Gets the TIA range parameters.
NT_GetTIArangeParams = lib.NT_GetTIArangeParams
NT_GetTIArangeParams.restype = c_short
NT_GetTIArangeParams.argtypes = [POINTER(c_char), NT_TIARangeParameters, KNA_TIARangeParameters]


# Gets the tracking threshold signal.
NT_GetTrackingThresholdSignal = lib.NT_GetTrackingThresholdSignal
NT_GetTrackingThresholdSignal.restype = c_float
NT_GetTrackingThresholdSignal.argtypes = [POINTER(c_char)]


# Queries if the time since the last message has exceeded the lastMsgTimeout set by NT_EnableLastMsgTimer(char const * serialNo, bool enable, __int32 lastMsgTimeout ).
NT_HasLastMsgTimerOverrun = lib.NT_HasLastMsgTimerOverrun
NT_HasLastMsgTimerOverrun.restype = c_bool
NT_HasLastMsgTimerOverrun.argtypes = [POINTER(c_char)]


# Move the scan circle to the home position.
NT_HomeCircle = lib.NT_HomeCircle
NT_HomeCircle.restype = c_short
NT_HomeCircle.argtypes = [POINTER(c_char)]


# Sends a command to the device to make it identify iteself.
NT_Identify = lib.NT_Identify
NT_Identify.restype = c_void_p
NT_Identify.argtypes = [POINTER(c_char)]


# Get the channel enabled state.
NT_IsChannelEnabled = lib.NT_IsChannelEnabled
NT_IsChannelEnabled.restype = c_bool
NT_IsChannelEnabled.argtypes = [POINTER(c_char), c_long]


# Update device with named settings.
NT_LoadNamedSettings = lib.NT_LoadNamedSettings
NT_LoadNamedSettings.restype = c_bool
NT_LoadNamedSettings.argtypes = [POINTER(c_char), POINTER(c_char)]


# Update device with stored settings.
NT_LoadSettings = lib.NT_LoadSettings
NT_LoadSettings.restype = c_bool
NT_LoadSettings.argtypes = [POINTER(c_char)]


# Gets the MessageQueue size.
NT_MessageQueueSize = lib.NT_MessageQueueSize
NT_MessageQueueSize.restype = c_int
NT_MessageQueueSize.argtypes = [POINTER(c_char)]


# Open the device for communications.
NT_Open = lib.NT_Open
NT_Open.restype = c_short
NT_Open.argtypes = [POINTER(c_char)]


# Gets the polling loop duration.
NT_PollingDuration = lib.NT_PollingDuration
NT_PollingDuration.restype = c_long
NT_PollingDuration.argtypes = [POINTER(c_char)]


# Registers a callback on the message queue.
NT_RegisterMessageCallback = lib.NT_RegisterMessageCallback
NT_RegisterMessageCallback.restype = c_void_p
NT_RegisterMessageCallback.argtypes = [POINTER(c_char), c_void_p]


# Request the channel states from the device.
NT_RequestChannelStates = lib.NT_RequestChannelStates
NT_RequestChannelStates.restype = c_short
NT_RequestChannelStates.argtypes = [POINTER(c_char)]


# Requests the scan circle diameter Lookup Table (LUT).
NT_RequestCircleDiameterLUT = lib.NT_RequestCircleDiameterLUT
NT_RequestCircleDiameterLUT.restype = c_short
NT_RequestCircleDiameterLUT.argtypes = [POINTER(c_char)]


# Requests the home position of the scan circle.
NT_RequestCircleHomePosition = lib.NT_RequestCircleHomePosition
NT_RequestCircleHomePosition.restype = c_short
NT_RequestCircleHomePosition.argtypes = [POINTER(c_char)]


# Requests the scanning circle parameters.
NT_RequestCircleParams = lib.NT_RequestCircleParams
NT_RequestCircleParams.restype = c_short
NT_RequestCircleParams.argtypes = [POINTER(c_char)]


# Requests the current scan circle centre position.
NT_RequestCirclePosition = lib.NT_RequestCirclePosition
NT_RequestCirclePosition.restype = c_short
NT_RequestCirclePosition.argtypes = [POINTER(c_char)]


# Request the NanoTrak control mode.
NT_RequestControlMode = lib.NT_RequestControlMode
NT_RequestControlMode.restype = c_short
NT_RequestControlMode.argtypes = [POINTER(c_char)]


# Requests the NanoTrak Feedback Source.
NT_RequestFeedbackSource = lib.NT_RequestFeedbackSource
NT_RequestFeedbackSource.restype = c_short
NT_RequestFeedbackSource.argtypes = [POINTER(c_char)]


# Requests the control loop gain.
NT_RequestGain = lib.NT_RequestGain
NT_RequestGain.restype = c_short
NT_RequestGain.argtypes = [POINTER(c_char)]


# Requests the MaxTravel for the Piezos in um.
NT_RequestMaxTravel = lib.NT_RequestMaxTravel
NT_RequestMaxTravel.restype = c_short
NT_RequestMaxTravel.argtypes = [POINTER(c_char)]


# Requests the NanoTrak mode.
NT_RequestMode = lib.NT_RequestMode
NT_RequestMode.restype = c_short
NT_RequestMode.argtypes = [POINTER(c_char)]


# Request the device updates the NanoTrak channel numbers.
NT_RequestNTChannels = lib.NT_RequestNTChannels
NT_RequestNTChannels.restype = c_short
NT_RequestNTChannels.argtypes = [POINTER(c_char)]


# Requests the phase compensation parameters.
NT_RequestPhaseCompensationParams = lib.NT_RequestPhaseCompensationParams
NT_RequestPhaseCompensationParams.restype = c_short
NT_RequestPhaseCompensationParams.argtypes = [POINTER(c_char)]


# Requests a TIA reading.
NT_RequestReading = lib.NT_RequestReading
NT_RequestReading.restype = c_short
NT_RequestReading.argtypes = [POINTER(c_char)]


# Requests that all settings are download from device.
NT_RequestSettings = lib.NT_RequestSettings
NT_RequestSettings.restype = c_short
NT_RequestSettings.argtypes = [POINTER(c_char)]


# Requests the NanoTrak signal state.
NT_RequestSignalState = lib.NT_RequestSignalState
NT_RequestSignalState.restype = c_short
NT_RequestSignalState.argtypes = [POINTER(c_char)]


# Requests the status bits and reading.
NT_RequestStatus = lib.NT_RequestStatus
NT_RequestStatus.restype = c_short
NT_RequestStatus.argtypes = [POINTER(c_char)]


# Request the status bits which identify the current device state.
NT_RequestStatusBits = lib.NT_RequestStatusBits
NT_RequestStatusBits.restype = c_short
NT_RequestStatusBits.argtypes = [POINTER(c_char)]


# Requests the NanoTrak tracking threshold signal.
NT_RequestTIALPFilterParams = lib.NT_RequestTIALPFilterParams
NT_RequestTIALPFilterParams.restype = c_short
NT_RequestTIALPFilterParams.argtypes = [POINTER(c_char)]


# Requests the TIA range parameters.
NT_RequestTIArangeParams = lib.NT_RequestTIArangeParams
NT_RequestTIArangeParams.restype = c_short
NT_RequestTIArangeParams.argtypes = [POINTER(c_char)]


# Requests the NanoTrak tracking threshold signal.
NT_RequestTrackingThresholdSignal = lib.NT_RequestTrackingThresholdSignal
NT_RequestTrackingThresholdSignal.restype = c_short
NT_RequestTrackingThresholdSignal.argtypes = [POINTER(c_char)]


# Sets the scan circle diameter.
NT_SetCircleDiameter = lib.NT_SetCircleDiameter
NT_SetCircleDiameter.restype = c_short
NT_SetCircleDiameter.argtypes = [POINTER(c_char), c_long]


# Sets the scan circle diameter Lookup Table (LUT).
NT_SetCircleDiameterLUT = lib.NT_SetCircleDiameterLUT
NT_SetCircleDiameterLUT.restype = c_short
NT_SetCircleDiameterLUT.argtypes = [POINTER(c_char), NT_CircleDiameterLUT]


# Sets the home position of the scan circle.
NT_SetCircleHomePosition = lib.NT_SetCircleHomePosition
NT_SetCircleHomePosition.restype = c_short
NT_SetCircleHomePosition.argtypes = [POINTER(c_char), NT_HVComponent]


# Sets the scanning circle parameters.
NT_SetCircleParams = lib.NT_SetCircleParams
NT_SetCircleParams.restype = c_short
NT_SetCircleParams.argtypes = [POINTER(c_char), NT_CircleParameters]


# Set the NanoTrak control mode.
NT_SetControlMode = lib.NT_SetControlMode
NT_SetControlMode.restype = c_short
NT_SetControlMode.argtypes = [POINTER(c_char), c_long, NT_ControlMode]


# Sets the NanoTrak feedback source.
NT_SetFeedbackSource = lib.NT_SetFeedbackSource
NT_SetFeedbackSource.restype = c_short
NT_SetFeedbackSource.argtypes = [POINTER(c_char), NT_FeedbackSource, KNA_FeedbackSource]


# Sets the control loop gain.
NT_SetGain = lib.NT_SetGain
NT_SetGain.restype = c_short
NT_SetGain.argtypes = [POINTER(c_char), c_short]


# Setsthe nanoTrak operating mode.
NT_SetMode = lib.NT_SetMode
NT_SetMode.restype = c_short
NT_SetMode.argtypes = [POINTER(c_char), NT_Mode]


# Sets the NanoTrak channels to (usually) piezos.
NT_SetNTChannels = lib.NT_SetNTChannels
NT_SetNTChannels.restype = c_short
NT_SetNTChannels.argtypes = [POINTER(c_char), c_short, c_short]


# Sets the phase compensation parameters.
NT_SetPhaseCompensationParams = lib.NT_SetPhaseCompensationParams
NT_SetPhaseCompensationParams.restype = c_short
NT_SetPhaseCompensationParams.argtypes = [POINTER(c_char), NT_HVComponent]


# Get the TIA Range Mode and OddEven mode.
NT_SetRangeMode = lib.NT_SetRangeMode
NT_SetRangeMode.restype = c_short
NT_SetRangeMode.argtypes = [POINTER(c_char), NT_TIARangeMode, NT_OddOrEven]


# Sets the TIA long pass filter parameters.
NT_SetTIALPFilterParams = lib.NT_SetTIALPFilterParams
NT_SetTIALPFilterParams.restype = c_short
NT_SetTIALPFilterParams.argtypes = [POINTER(c_char), NT_LowPassFilterParameters]


# Sets TIA range.
NT_SetTIARange = lib.NT_SetTIARange
NT_SetTIARange.restype = c_short
NT_SetTIARange.argtypes = [POINTER(c_char), NT_TIARange, KNA_TIARange]


# Sets the TIA range parameters.
NT_SetTIArangeParams = lib.NT_SetTIArangeParams
NT_SetTIArangeParams.restype = c_short
NT_SetTIArangeParams.argtypes = [POINTER(c_char), NT_TIARangeParameters, KNA_TIARangeParameters]


# Sets the tracking threshold signal.
NT_SetTrackingThresholdSignal = lib.NT_SetTrackingThresholdSignal
NT_SetTrackingThresholdSignal.restype = c_short
NT_SetTrackingThresholdSignal.argtypes = [POINTER(c_char), c_float]


# Starts the internal polling loop which continuously requests position and status.
NT_StartPolling = lib.NT_StartPolling
NT_StartPolling.restype = c_bool
NT_StartPolling.argtypes = [POINTER(c_char), c_int]


# Stops the internal polling loop.
NT_StopPolling = lib.NT_StopPolling
NT_StopPolling.restype = c_void_p
NT_StopPolling.argtypes = [POINTER(c_char)]


# Gets the time in milliseconds since tha last message was received from the device.
NT_TimeSinceLastMsgReceived = lib.NT_TimeSinceLastMsgReceived
NT_TimeSinceLastMsgReceived.restype = c_bool
NT_TimeSinceLastMsgReceived.argtypes = [POINTER(c_char), c_int64]




from tlk.benchtopbrushlessmotor import *
from tlk.benchtopdcservo import *
from tlk.benchtopnanotrack import *
from tlk.benchtoppiezo import *
from tlk.benchtoppiezopdxc2 import *
from tlk.benchtopprecisionpiezo import *
from tlk.benchtopsteppermotor import *
# from tlk.benchtopvoicecoil import *
from tlk.filterflipper import *
from tlk.integratedprecisionpiezo import *
from tlk.integratedsteppermotor import *
from tlk.kcubebrushlessmotor import *
# from tlk.kcubedcservo import *
from tlk.kcubeinertialmotor import *
from tlk.kcubelaserdiode import *
from tlk.kcubelasersource import *
from tlk.kcubenanotrack import *
from tlk.kcubepiezo import *
from tlk.kcubepiezostraingauge import *
from tlk.kcubepositionaligner import *
from tlk.kcubesolenoid import *
from tlk.kcubesteppermotor import *
# from tlk.modularnanotrack import *
# from tlk.modularsteppermotor import *
from tlk.polarizer import *
from tlk.tcubebrushlessmotor import *
from tlk.tcubeinertialmotor import *
from tlk.tcubelaserdiode import *
from tlk.tcubelasersource import *
from tlk.tcubenanotrack import *
from tlk.tcubepiezo import *
from tlk.tcubequad import *
from tlk.tcubeservo import *
from tlk.tcubesolenoid import *
from tlk.tcubesteppermotor import *
from tlk.tcubestraingauge import *
from tlk.tcubetec import *
